/// <reference types="node" />
import { AchievementProgressData, CharacterData, ServerData, ActionData, ChestOpenedData, ChestData, EntitiesData, GameResponseData, PartyData, WelcomeData, EntityData, PlayerData } from "./definitions/adventureland-server";
import { GData, SkillName, BankInfo, MapName, ItemInfo, ItemName, SlotType, MonsterName, SInfo, IPosition, NPCType, BankPackType, TradeSlotType } from "./definitions/adventureland";
import { NodeData } from "./definitions/pathfinder";
import { Observer } from "./Observer";
import { Entity } from "./Entity";
import { Player } from "./Player";
export declare class Character extends Observer {
    protected userID: string;
    protected userAuth: string;
    protected characterID: string;
    protected lastPositionUpdate: number;
    protected promises: Promise<boolean>[];
    protected pingNum: number;
    protected pingMap: Map<string, number>;
    protected timeouts: Map<string, NodeJS.Timeout>;
    achievements: Map<string, AchievementProgressData>;
    bank: BankInfo;
    character: CharacterData;
    chests: Map<string, ChestData>;
    entities: Map<string, Entity>;
    nextSkill: Map<SkillName, Date>;
    party: PartyData;
    pings: number[];
    players: Map<string, Player>;
    projectiles: Map<string, ActionData & {
        date: Date;
    }>;
    server: WelcomeData;
    S: SInfo;
    constructor(userID: string, userAuth: string, characterID: string, g: GData, serverData: ServerData);
    protected parseCharacter(data: CharacterData): void;
    protected parseEntities(data: EntitiesData): Promise<void>;
    protected parseGameResponse(data: GameResponseData): void;
    protected setNextSkill(skill: SkillName, next: Date): void;
    protected updatePositions(): void;
    connect(): Promise<void>;
    disconnect(): Promise<void>;
    requestEntitiesData(): Promise<EntitiesData>;
    requestPlayerData(): Promise<CharacterData>;
    sendPing(): string;
    acceptMagiport(name: string): Promise<NodeData>;
    acceptPartyInvite(id: string): Promise<PartyData>;
    acceptPartyRequest(id: string): Promise<PartyData>;
    attack(id: string): Promise<string>;
    buy(itemName: ItemName, quantity?: number): Promise<number>;
    buyFromMerchant(id: string, slot: TradeSlotType, rid: string, quantity?: number): Promise<void>;
    buyFromPonty(item: ItemInfo): unknown;
    calculateItemCost(item: ItemInfo): number;
    calculateItemGrade(item: ItemInfo): number;
    canBuy(item: ItemName): boolean;
    canKillInOneShot(entity: EntityData): boolean;
    canUse(skill: SkillName): boolean;
    compound(item1Pos: number, item2Pos: number, item3Pos: number, cscrollPos: number, offeringPos?: number): Promise<boolean>;
    craft(item: ItemName): Promise<void>;
    depositGold(gold: number): unknown;
    depositItem(inventoryPos: number, bankPack?: Exclude<BankPackType, "gold">, bankSlot?: number): unknown;
    equip(inventoryPos: number, equipSlot?: SlotType): Promise<void>;
    exchange(inventoryPos: number): Promise<boolean>;
    finishMonsterHuntQuest(): void;
    getMonsterHuntQuest(): Promise<void>;
    getPontyItems(): Promise<ItemInfo[]>;
    isFull(): boolean;
    leaveMap(): Promise<void>;
    leaveParty(): void;
    move(x: number, y: number, safetyCheck?: boolean): Promise<NodeData>;
    openChest(id: string): Promise<ChestOpenedData>;
    regenHP(): Promise<void>;
    regenMP(): Promise<void>;
    respawn(): void;
    scare(): Promise<string[]>;
    sell(itemPos: number, quantity?: number): Promise<void>;
    sendCM(to: string[], message: unknown): Promise<void>;
    sendGold(to: string, amount: number): Promise<number>;
    sendItem(to: string, inventoryPos: number, quantity?: number): Promise<void>;
    sendPartyInvite(id: string): void;
    sendPartyRequest(id: string): void;
    protected lastSmartMove: number;
    smartMove(to: MapName | MonsterName | NPCType | IPosition, options?: {
        getWithin?: number;
        useBlink?: boolean;
    }): Promise<NodeData>;
    stopSmartMove(): Promise<NodeData>;
    stopWarpToTown(): void;
    throwSnowball(target: string): Promise<string>;
    transport(map: MapName, spawn: number): Promise<void>;
    unequip(slot: SlotType | TradeSlotType): Promise<void>;
    upgrade(itemPos: number, scrollPos: number): Promise<boolean>;
    useHPPot(itemPos: number): Promise<void>;
    useMPPot(itemPos: number): Promise<void>;
    warpToJail(): Promise<NodeData>;
    warpToTown(): Promise<NodeData>;
    withdrawGold(gold: number): unknown;
    withdrawItem(bankPack: Exclude<BankPackType, "gold">, bankPos: number, inventoryPos?: number): unknown;
    countItem(item: ItemName, inventory?: ItemInfo[], args?: {
        levelGreaterThan?: number;
        levelLessThan?: number;
    }): number;
    getCooldown(skill: SkillName): number;
    getNearestMonster(mtype?: MonsterName): {
        monster: EntityData;
        distance: number;
    };
    getNearestAttackablePlayer(): {
        player: PlayerData;
        distance: number;
    };
    hasItem(iN: ItemName, inv?: ItemInfo[], args?: {
        levelGreaterThan?: number;
        levelLessThan?: number;
    }): boolean;
    isEquipped(itemName: ItemName): boolean;
    isPVP(): boolean;
    locateDuplicateItems(inventory?: ItemInfo[]): {
        [T in ItemName]?: number[];
    };
    locateItem(iN: ItemName, inv?: ItemInfo[], args?: {
        level?: number;
        levelGreaterThan?: number;
        levelLessThan?: number;
        locateHighestLevel?: number;
        quantityGreaterThan?: number;
    }): number;
    locateItems(itemName: ItemName, inventory?: ItemInfo[], filters?: {
        levelGreaterThan?: number;
        levelLessThan?: number;
    }): number[];
    locateMonsters(mType: MonsterName): NodeData[];
    locateNPCs(npcType: NPCType): NodeData[];
}
