"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Character = void 0;
const Constants_1 = require("./Constants");
const Observer_1 = require("./Observer");
const index_1 = require("./index");
const Tools_1 = require("./Tools");
const Entity_1 = require("./Entity");
const Player_1 = require("./Player");
class Character extends Observer_1.Observer {
    constructor(userID, userAuth, characterID, g, serverData) {
        super(serverData, g);
        this.promises = [];
        this.pingNum = 1;
        this.pingMap = new Map();
        this.timeouts = new Map();
        this.achievements = new Map();
        this.bank = { gold: 0 };
        this.chests = new Map();
        this.entities = new Map();
        this.nextSkill = new Map();
        this.pings = [];
        this.players = new Map();
        this.projectiles = new Map();
        this.lastSmartMove = Date.now();
        this.userID = userID;
        this.userAuth = userAuth;
        this.characterID = characterID;
        this.socket.on("start", (data) => {
            this.parseCharacter(data);
            if (data.entities)
                this.parseEntities(data.entities);
            this.S = data.s_info;
        });
        this.socket.on("achievement_progress", (data) => {
            this.achievements.set(data.name, data);
            console.log(data);
        });
        this.socket.on("action", (data) => {
            this.projectiles.set(data.pid, { ...data, date: new Date() });
        });
        this.socket.on("chest_opened", (data) => {
            this.chests.delete(data.id);
        });
        this.socket.on("death", (data) => {
            const entity = this.entities.get(data.id);
            if (this.S && entity && this.S[entity.type])
                delete this.S[entity.type];
            this.entities.delete(data.id);
        });
        this.socket.on("disappear", (data) => {
            if (data.reason == "disconnect") {
                this.players.delete(data.id);
            }
            else if (data.reason == "transport" && (typeof data.s == "number" || typeof data.s == "undefined")) {
                const player = this.players.get(data.id);
                if (player) {
                    const location = this.G.maps[data.to].spawns[data.s == undefined ? 0 : data.s];
                    player.x = location[0];
                    player.y = location[1];
                    this.players.set(data.id, player);
                }
            }
            else if (data.reason == undefined) {
                this.entities.delete(data.id);
            }
            else if (data.reason == "invis") {
                this.players.delete(data.id);
                this.entities.delete(data.id);
            }
            else {
                const player = this.players.get(data.id);
                if (player) {
                    if (Array.isArray(data.s)) {
                        player.x = data.s[0];
                        player.y = data.s[1];
                    }
                    else {
                        const location = this.G.maps[data.to].spawns[data.s == undefined ? 0 : data.s];
                        player.x = location[0];
                        player.y = location[1];
                    }
                    this.players.set(data.id, player);
                }
                else {
                    const entity = this.entities.get(data.id);
                    if (entity) {
                        if (Array.isArray(data.s)) {
                            entity.x = data.s[0];
                            entity.y = data.s[1];
                        }
                        else {
                            const location = this.G.maps[data.to].spawns[data.s == undefined ? 0 : data.s];
                            entity.x = location[0];
                            entity.y = location[1];
                        }
                        this.entities.set(data.id, entity);
                    }
                }
            }
            this.updatePositions();
        });
        this.socket.on("disconnect", () => {
        });
        this.socket.on("drop", (data) => {
            this.chests.set(data.id, data);
        });
        this.socket.on("entities", (data) => {
            this.parseEntities(data);
        });
        this.socket.on("eval", (data) => {
            const skillReg1 = /skill_timeout\s*\(\s*['"](.+?)['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.exec(data.code);
            if (skillReg1) {
                const skill = skillReg1[1];
                let cooldown;
                if (skillReg1[2]) {
                    cooldown = Number.parseFloat(skillReg1[2]);
                }
                else if (this.G.skills[skill].cooldown) {
                    cooldown = this.G.skills[skill].cooldown;
                }
                this.setNextSkill(skill, new Date(Date.now() + Math.ceil(cooldown)));
                return;
            }
            const potReg = /pot_timeout\s*\(\s*(\d+\.?\d+?)\s*\)/.exec(data.code);
            if (potReg) {
                const cooldown = Number.parseFloat(potReg[1]);
                this.setNextSkill("use_hp", new Date(Date.now() + Math.ceil(cooldown)));
                this.setNextSkill("use_mp", new Date(Date.now() + Math.ceil(cooldown)));
                return;
            }
        });
        this.socket.on("game_error", (data) => {
            if (typeof data == "string") {
                console.error(`Game Error: ${data}`);
            }
            else {
                console.error("Game Error ----------");
                console.error(data);
            }
            this.disconnect();
        });
        this.socket.on("game_response", (data) => {
            this.parseGameResponse(data);
        });
        this.socket.on("hit", (data) => {
            if (data.miss || data.evade) {
                this.projectiles.delete(data.pid);
                return;
            }
            if (data.reflect) {
                this.projectiles.get(data.pid);
            }
            if (data.kill == true) {
                this.projectiles.delete(data.pid);
                this.entities.delete(data.id);
            }
            else if (data.damage) {
                this.projectiles.delete(data.pid);
                const entity = this.entities.get(data.id);
                if (entity) {
                    entity.hp = entity.hp - data.damage;
                    this.entities.set(data.id, entity);
                }
            }
        });
        this.socket.on("new_map", (data) => {
            this.projectiles.clear();
            this.parseEntities(data.entities);
            if (this.character) {
                this.character.x = data.x;
                this.character.y = data.y;
                this.character.in = data.in;
                this.character.map = data.name;
                this.character.m = data.m;
            }
        });
        this.socket.on("party_update", (data) => {
            this.party = data;
        });
        this.socket.on("ping_ack", (data) => {
            if (this.pingMap.has(data.id)) {
                const ping = Date.now() - this.pingMap.get(data.id);
                this.pings.push(ping);
                console.log(`Ping: ${ping}`);
                if (this.pings.length > Constants_1.Constants.MAX_PINGS)
                    this.pings.shift();
                this.pingMap.delete(data.id);
            }
        });
        this.socket.on("player", (data) => {
            this.parseCharacter(data);
        });
        this.socket.on("q_data", (data) => {
            if (data.q.upgrade)
                this.character.q.upgrade = data.q.upgrade;
            if (data.q.compound)
                this.character.q.compound = data.q.compound;
        });
        this.socket.on("server_info", (data) => {
            for (const mtype in data) {
                if (typeof data[mtype] !== "object")
                    continue;
                const mN = mtype;
                if (data[mN].live && data[mN].hp == undefined) {
                    data[mN].hp = this.G.monsters[mN].hp;
                    data[mN].max_hp = this.G.monsters[mN].hp;
                }
            }
            this.S = data;
        });
        this.socket.on("upgrade", (data) => {
            if (data.type == "compound" && this.character.q.compound)
                delete this.character.q.compound;
            else if (data.type == "upgrade" && this.character.q.upgrade)
                delete this.character.q.upgrade;
        });
        this.socket.on("welcome", (data) => {
            this.server = data;
            if (this.serverRegion !== data.region)
                console.error(`REGIONS DO NOT MATCH!? ${this.serverRegion} VS. ${data.region}`);
            if (this.serverIdentifier !== data.name)
                console.error(`IDENTIFIERS DO NOT MATCH!? ${this.serverIdentifier} VS. ${data.name}`);
            this.serverRegion = data.region;
            this.serverIdentifier = data.name;
            this.socket.emit("loaded", {
                height: 1080,
                width: 1920,
                scale: 2,
                success: 1
            });
        });
    }
    parseCharacter(data) {
        if (!this.character) {
            this.character = data;
            delete this.character["base_gold"];
            delete this.character["entities"];
            delete this.character["hitchhikers"];
            delete this.character["info"];
            delete this.character["s_info"];
            delete this.character["user"];
            this.character.going_x = data.x;
            this.character.going_y = data.y;
            this.character.moving = false;
            this.character.damage_type = this.G.classes[data.ctype].damage_type;
        }
        for (const datum in data) {
            if (datum == "hitchhikers") {
                for (const [event, datum] of data.hitchhikers) {
                    if (event == "game_response") {
                        this.parseGameResponse(datum);
                    }
                }
            }
            else if (datum == "moving") {
            }
            else if (datum == "tp") {
            }
            else if (datum == "user") {
                this.bank = data.user;
            }
            else {
                this.character[datum] = data[datum];
            }
        }
    }
    async parseEntities(data) {
        if (data.type == "all") {
            this.entities.clear();
            this.players.clear();
        }
        else if (this.character) {
            this.updatePositions();
        }
        for (const monster of data.monsters) {
            if (!this.entities.has(monster.id)) {
                const e = new Entity_1.Entity(monster, data.map, this.G);
                this.entities.set(monster.id, e);
            }
            else {
                const e = this.entities.get(monster.id);
                e.updateData(monster);
            }
        }
        for (const player of data.players) {
            if (!this.players.has(player.id)) {
                const p = new Player_1.Player(player, data.map, this.G);
                this.players.set(player.id, p);
            }
            else {
                const p = this.players.get(player.id);
                p.updateData(player);
            }
        }
    }
    parseGameResponse(data) {
        if (typeof (data) == "object") {
            if (data.response == "cooldown") {
                const skill = data.skill;
                if (skill) {
                    const cooldown = data.ms;
                    this.setNextSkill(skill, new Date(Date.now() + Math.ceil(cooldown)));
                }
            }
            else if (data.response == "ex_condition") {
                delete this.character.s[data.name];
            }
            else if (data.response == "skill_success") {
                const cooldown = this.G.skills[data.name].cooldown;
                if (cooldown) {
                    this.setNextSkill(data.name, new Date(Date.now() + cooldown));
                }
            }
            else {
                console.info("Game Response Data -----");
                console.info(data);
            }
        }
        else if (typeof (data) == "string") {
            console.info(`Game Response: ${data}`);
        }
    }
    setNextSkill(skill, next) {
        this.nextSkill.set(skill, next);
    }
    updatePositions() {
        if (this.lastPositionUpdate) {
            const msSinceLastUpdate = Date.now() - this.lastPositionUpdate;
            for (const entity of this.entities.values()) {
                if (!entity.moving)
                    continue;
                let speed = entity.speed;
                if (entity.target !== undefined && entity.charge !== undefined)
                    speed = entity.charge;
                const distanceTravelled = speed * msSinceLastUpdate / 1000;
                const angle = Math.atan2(entity.going_y - entity.y, entity.going_x - entity.x);
                const distanceToGoal = Tools_1.Tools.distance({ x: entity.x, y: entity.y }, { x: entity.going_x, y: entity.going_y });
                if (distanceTravelled > distanceToGoal) {
                    entity.moving = false;
                    entity.x = entity.going_x;
                    entity.y = entity.going_y;
                }
                else {
                    entity.x = entity.x + Math.cos(angle) * distanceTravelled;
                    entity.y = entity.y + Math.sin(angle) * distanceTravelled;
                }
                for (const condition in entity.s) {
                    const newCooldown = entity.s[condition].ms - msSinceLastUpdate;
                    if (newCooldown <= 0)
                        delete entity.s[condition];
                    else
                        entity.s[condition].ms = newCooldown;
                }
            }
            for (const player of this.players.values()) {
                if (!player.moving)
                    continue;
                const distanceTravelled = player.speed * msSinceLastUpdate / 1000;
                const angle = Math.atan2(player.going_y - player.y, player.going_x - player.x);
                const distanceToGoal = Tools_1.Tools.distance({ x: player.x, y: player.y }, { x: player.going_x, y: player.going_y });
                if (distanceTravelled > distanceToGoal) {
                    player.moving = false;
                    player.x = player.going_x;
                    player.y = player.going_y;
                }
                else {
                    player.x = player.x + Math.cos(angle) * distanceTravelled;
                    player.y = player.y + Math.sin(angle) * distanceTravelled;
                }
                for (const condition in player.s) {
                    const newCooldown = player.s[condition].ms - msSinceLastUpdate;
                    if (newCooldown <= 0)
                        delete player.s[condition];
                    else
                        player.s[condition].ms = newCooldown;
                }
            }
            if (this.character.moving) {
                const distanceTravelled = this.character.speed * msSinceLastUpdate / 1000;
                const angle = Math.atan2(this.character.going_y - this.character.y, this.character.going_x - this.character.x);
                const distanceToGoal = Tools_1.Tools.distance({ x: this.character.x, y: this.character.y }, { x: this.character.going_x, y: this.character.going_y });
                if (distanceTravelled > distanceToGoal) {
                    this.character.moving = false;
                    this.character.x = this.character.going_x;
                    this.character.y = this.character.going_y;
                }
                else {
                    this.character.x = this.character.x + Math.cos(angle) * distanceTravelled;
                    this.character.y = this.character.y + Math.sin(angle) * distanceTravelled;
                }
            }
            for (const condition in this.character.s) {
                const newCooldown = this.character.s[condition].ms - msSinceLastUpdate;
                if (newCooldown <= 0)
                    delete this.character.s[condition];
                else
                    this.character.s[condition].ms = newCooldown;
            }
        }
        let toDelete = [];
        for (const [id, entity] of this.entities) {
            if (Tools_1.Tools.distance(this.character, entity) < 600)
                continue;
            toDelete.push(id);
        }
        for (const id of toDelete)
            this.entities.delete(id);
        toDelete = [];
        for (const [id, player] of this.players) {
            if (Tools_1.Tools.distance(this.character, player) < 600)
                continue;
            toDelete.push(id);
        }
        for (const id of toDelete)
            this.players.delete(id);
        this.lastPositionUpdate = Date.now();
    }
    async connect() {
        const connected = new Promise((resolve, reject) => {
            const failCheck = (data) => {
                if (typeof data == "string") {
                    reject(`Failed to connect: ${data}`);
                }
                else {
                    reject(`Failed to connect: ${data.message}`);
                }
            };
            const startCheck = () => {
                resolve();
            };
            setTimeout(() => {
                this.socket.removeListener("start", startCheck);
                this.socket.removeListener("game_error", failCheck);
                reject("Failed to start within 10s.");
            }, 10000);
            this.socket.once("start", startCheck);
            this.socket.once("game_error", failCheck);
        });
        this.socket.on("welcome", () => {
            this.socket.emit("auth", {
                auth: this.userAuth,
                character: this.characterID,
                height: 1080,
                no_graphics: "True",
                no_html: "1",
                passphrase: "",
                scale: 2,
                user: this.userID,
                width: 1920
            });
        });
        this.socket.open();
        return connected;
    }
    async disconnect() {
        if (this.socket.disconnected)
            return;
        console.warn("Disconnecting!");
        this.socket.close();
        this.socket.removeAllListeners();
        for (const timer of this.timeouts.values())
            clearTimeout(timer);
    }
    async requestEntitiesData() {
        return new Promise((resolve, reject) => {
            const checkEntitiesEvent = (data) => {
                this.socket.removeListener("entities");
                if (data.type == "all")
                    resolve(data);
            };
            setTimeout(() => {
                this.socket.removeListener("entities", checkEntitiesEvent);
                reject(`requestEntitiesData timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("entities", checkEntitiesEvent);
            this.socket.emit("send_updates", {});
        });
    }
    async requestPlayerData() {
        return new Promise((resolve, reject) => {
            const checkPlayerEvent = (data) => {
                if (data.s.typing) {
                    this.socket.removeListener("player", checkPlayerEvent);
                    resolve(data);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkPlayerEvent);
                reject(`requestPlayerData timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkPlayerEvent);
            this.socket.emit("property", { typing: true });
        });
    }
    sendPing() {
        const pingID = this.pingNum.toString();
        this.pingNum++;
        this.pingMap.set(pingID, Date.now());
        this.socket.emit("ping_trig", { id: pingID });
        return pingID;
    }
    acceptMagiport(name) {
        const acceptedMagiport = new Promise((resolve, reject) => {
            const magiportCheck = (data) => {
                if (data.effect == "magiport") {
                    this.socket.removeListener("new_map", magiportCheck);
                    resolve({ map: data.in, x: data.x, y: data.y });
                }
            };
            setTimeout(() => {
                this.socket.removeListener("new_map", magiportCheck);
                reject(`acceptMagiport timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("new_map", magiportCheck);
        });
        this.socket.emit("magiport", { name: name });
        return acceptedMagiport;
    }
    acceptPartyInvite(id) {
        const acceptedInvite = new Promise((resolve, reject) => {
            const partyCheck = (data) => {
                if (data.list.includes(this.character.id)
                    && data.list.includes(id)) {
                    this.socket.removeListener("party_update", partyCheck);
                    this.socket.removeListener("game_log", unableCheck);
                    resolve(data);
                }
            };
            const unableCheck = (data) => {
                const notFound = RegExp("^.+? is not found$");
                if (data == "Invitation expired") {
                    this.socket.removeListener("party_update", partyCheck);
                    this.socket.removeListener("game_log", unableCheck);
                    reject(data);
                }
                else if (notFound.test(data)) {
                    this.socket.removeListener("party_update", partyCheck);
                    this.socket.removeListener("game_log", unableCheck);
                    reject(data);
                }
                else if (data == "Already partying") {
                    if (this.party.list.includes(this.character.id)
                        && this.party.list.includes(id)) {
                        this.socket.removeListener("party_update", partyCheck);
                        this.socket.removeListener("game_log", unableCheck);
                        resolve(this.party);
                    }
                    else {
                        this.socket.removeListener("party_update", partyCheck);
                        this.socket.removeListener("game_log", unableCheck);
                        reject(data);
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("party_update", partyCheck);
                this.socket.removeListener("game_log", unableCheck);
                reject(`acceptPartyInvite timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("party_update", partyCheck);
            this.socket.on("game_log", unableCheck);
        });
        this.socket.emit("party", { event: "accept", name: id });
        return acceptedInvite;
    }
    acceptPartyRequest(id) {
        const acceptedRequest = new Promise((resolve, reject) => {
            const partyCheck = (data) => {
                if (data.list.includes(this.character.id)
                    && data.list.includes(id)) {
                    this.socket.removeListener("party_update", partyCheck);
                    resolve(data);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("party_update", partyCheck);
                reject(`acceptPartyRequest timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("party_update", partyCheck);
        });
        this.socket.emit("party", { event: "raccept", name: id });
        return acceptedRequest;
    }
    attack(id) {
        if (this.character.mp_cost > this.character.mp)
            return Promise.reject("Not enough MP to attack");
        const attackStarted = new Promise((resolve, reject) => {
            const deathCheck = (data) => {
                if (data.id == id) {
                    this.socket.removeListener("action", attackCheck);
                    this.socket.removeListener("game_response", failCheck);
                    this.socket.removeListener("death", deathCheck);
                    reject(`Entity ${id} not found`);
                }
            };
            const failCheck = (data) => {
                if (typeof data == "object") {
                    if (data.response == "disabled") {
                        this.socket.removeListener("action", attackCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("death", deathCheck);
                        reject(`Attack on ${id} failed (disabled).`);
                    }
                    else if (data.response == "attack_failed" && data.id == id) {
                        this.socket.removeListener("action", attackCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("death", deathCheck);
                        reject(`Attack on ${id} failed.`);
                    }
                    else if (data.response == "too_far" && data.id == id) {
                        this.socket.removeListener("action", attackCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("death", deathCheck);
                        reject(`${id} is too far away to attack (dist: ${data.dist}).`);
                    }
                    else if (data.response == "cooldown" && data.id == id) {
                        this.socket.removeListener("action", attackCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("death", deathCheck);
                        reject(`Attack on ${id} failed due to cooldown (ms: ${data.ms}).`);
                    }
                    else if (data.response == "no_mp" && data.place == "attack") {
                        this.socket.removeListener("action", attackCheck);
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("death", deathCheck);
                        reject(`Attack on ${id} failed due to insufficient MP.`);
                    }
                }
            };
            const attackCheck = (data) => {
                if (data.attacker == this.character.id && data.target == id && data.type == "attack") {
                    this.socket.removeListener("action", attackCheck);
                    this.socket.removeListener("game_response", failCheck);
                    this.socket.removeListener("death", deathCheck);
                    resolve(data.pid);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("action", attackCheck);
                this.socket.removeListener("game_response", failCheck);
                this.socket.removeListener("death", deathCheck);
                reject(`attack timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("game_response", failCheck);
            this.socket.on("death", deathCheck);
        });
        this.socket.emit("attack", { id: id });
        return attackStarted;
    }
    buy(itemName, quantity = 1) {
        if (this.character.gold < this.G.items[itemName].g)
            return Promise.reject(`Insufficient gold. We have ${this.character.gold}, but the item costs ${this.G.items[itemName].g}`);
        const itemReceived = new Promise((resolve, reject) => {
            const buyCheck1 = (data) => {
                if (!data.hitchhikers)
                    return;
                for (const hitchhiker of data.hitchhikers) {
                    if (hitchhiker[0] == "game_response") {
                        const data = hitchhiker[1];
                        if (typeof data == "object"
                            && data.response == "buy_success"
                            && data.name == itemName
                            && data.q == quantity) {
                            this.socket.removeListener("player", buyCheck1);
                            this.socket.removeListener("game_response", buyCheck2);
                            resolve(data.num);
                        }
                    }
                }
            };
            const buyCheck2 = (data) => {
                if (data == "buy_cant_npc") {
                    this.socket.removeListener("player", buyCheck1);
                    this.socket.removeListener("game_response", buyCheck2);
                    reject(`Cannot buy ${quantity} ${itemName}(s) from an NPC`);
                }
                else if (data == "buy_cant_space") {
                    this.socket.removeListener("player", buyCheck1);
                    this.socket.removeListener("game_response", buyCheck2);
                    reject(`Not enough inventory space to buy ${quantity} ${itemName}(s)`);
                }
                else if (data == "buy_cost") {
                    this.socket.removeListener("player", buyCheck1);
                    this.socket.removeListener("game_response", buyCheck2);
                    reject(`Not enough gold to buy ${quantity} ${itemName}(s)`);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", buyCheck1);
                this.socket.removeListener("game_response", buyCheck2);
                reject(`buy timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", buyCheck1);
            this.socket.on("game_response", buyCheck2);
        });
        if (this.G.items[itemName].s) {
            this.socket.emit("buy", { name: itemName, quantity: quantity });
        }
        else {
            this.socket.emit("buy", { name: itemName });
        }
        return itemReceived;
    }
    buyFromMerchant(id, slot, rid, quantity = 1) {
        if (quantity <= 0)
            return Promise.reject(`We can not buy a quantity of ${quantity}.`);
        const merchant = this.players.get(id);
        if (!merchant)
            return Promise.reject(`We can not see ${id} nearby.`);
        if (Tools_1.Tools.distance(this.character, merchant) > Constants_1.Constants.NPC_INTERACTION_DISTANCE)
            return Promise.reject(`We are too far away from ${id} to buy from.`);
        const item = merchant.slots[slot];
        if (!item)
            return Promise.reject(`We could not find an item in slot ${slot} on ${id}.`);
        if (item.b)
            return Promise.reject("The item is not for sale, this merchant is *buying* that item.");
        if (item.rid !== rid)
            return Promise.reject(`The RIDs do not match (item: ${item.rid}, supplied: ${rid})`);
        if (!merchant.slots[slot].q && quantity != 1) {
            console.warn("We are only going to buy 1, as there is only 1 available.");
            quantity = 1;
        }
        else if (merchant.slots[slot].q && quantity > merchant.slots[slot].q) {
            console.warn(`We can't buy ${quantity}, we can only buy ${merchant.slots[slot].q}, so we're doing that.`);
            quantity = merchant.slots[slot].q;
        }
        if (this.character.gold < merchant.slots[slot].price * quantity) {
            if (this.character.gold < merchant.slots[slot].price)
                return Promise.reject(`We don't have enough gold. It costs ${merchant.slots[slot].price}, but we only have ${this.character.gold}`);
            const buyableQuantity = Math.floor(this.character.gold / merchant.slots[slot].price);
            console.warn(`We don't have enough gold to buy ${quantity}, we can only buy ${buyableQuantity}, so we're doing that.`);
            quantity = buyableQuantity;
        }
        this.socket.emit("trade_buy", { slot: slot, id: id, rid: rid, q: quantity.toString() });
    }
    buyFromPonty(item) {
        if (!item.rid)
            return Promise.reject("This item does not have an 'rid'.");
        const price = this.G.items[item.name].g * (item.q ? item.q : 1);
        if (price > this.character.gold)
            return Promise.reject("We don't have enough gold to buy this.");
        this.socket.emit("sbuy", { rid: item.rid });
    }
    calculateItemCost(item) {
        const gInfo = this.G.items[item.name];
        let cost = gInfo.g;
        if (gInfo.compound) {
            for (let i = 0; i < item.level; i++) {
                cost *= 3;
                let scrollLevel = 0;
                for (const grade of gInfo.grades) {
                    if (i + 1 < grade) {
                        const scrollInfo = this.G.items[`cscroll${scrollLevel}`];
                        cost += scrollInfo.g;
                        break;
                    }
                    scrollLevel++;
                }
            }
        }
        else if (gInfo.upgrade) {
            for (let i = 0; i < item.level; i++) {
                let scrollLevel = 0;
                for (const grade of gInfo.grades) {
                    if (i + 1 < grade) {
                        const scrollInfo = this.G.items[`scroll${scrollLevel}`];
                        cost += scrollInfo.g;
                        break;
                    }
                    scrollLevel++;
                }
            }
        }
        return cost;
    }
    calculateItemGrade(item) {
        const gInfo = this.G.items[item.name];
        if (!gInfo.grades)
            return;
        let level = 0;
        for (const grade of gInfo.grades) {
            if (item.level < grade)
                break;
            level++;
        }
        return level;
    }
    canBuy(item) {
        if (this.isFull())
            return false;
        const gInfo = this.G.items[item];
        if (this.character.gold < gInfo.g)
            return false;
        const computerAvailable = this.locateItem("computer") !== undefined;
        let buyable = gInfo.buy;
        let close = false;
        if (buyable === undefined) {
            for (const map in this.G.maps) {
                if (buyable !== undefined)
                    break;
                if (!computerAvailable && map !== this.character.map)
                    continue;
                if (this.G.maps[map].ignore)
                    continue;
                for (const npc of this.G.maps[map].npcs) {
                    if (buyable !== undefined)
                        break;
                    if (this.G.npcs[npc.id].items === undefined)
                        continue;
                    for (const i of this.G.npcs[npc.id].items) {
                        if (i == item) {
                            buyable = true;
                            if (Tools_1.Tools.distance(this.character, { map: map, x: npc.position[0], y: npc.position[1] }) < Constants_1.Constants.NPC_INTERACTION_DISTANCE)
                                close = true;
                            break;
                        }
                    }
                }
            }
        }
        if (!buyable)
            return false;
        if (computerAvailable || close)
            return true;
        return false;
    }
    canKillInOneShot(entity) {
        const gInfo = this.G.monsters[entity.type];
        if (gInfo.abilities && gInfo.abilities.self_healing)
            return false;
        if (this.character.damage_type == "magical" && entity.reflection !== undefined)
            return false;
        if (this.character.damage_type == "physical" && entity.evasion !== undefined)
            return false;
        if (entity["1hp"]) {
            if (entity.hp == 1)
                return true;
            else
                return false;
        }
        return Tools_1.Tools.calculateDamageRange(this.character, entity)[0] > entity.hp;
    }
    canUse(skill) {
        if (this.character.rip)
            return false;
        if (this.character.s.stoned)
            return false;
        if (this.getCooldown(skill) > 0)
            return false;
        const gInfoSkill = this.G.skills[skill];
        if (gInfoSkill.mp !== undefined && this.character.mp < gInfoSkill.mp)
            return false;
        if (skill == "attack" && this.character.mp < this.character.mp_cost)
            return false;
        if (gInfoSkill.level !== undefined && this.character.level < gInfoSkill.level)
            return false;
        if (gInfoSkill.wtype) {
            if (!this.character.slots.mainhand)
                return false;
            const gInfoWeapon = this.G.items[this.character.slots.mainhand.name];
            if (typeof gInfoSkill.wtype == "object") {
                let isAcceptableWeapon = false;
                for (const wtype of gInfoSkill.wtype) {
                    if (gInfoWeapon.wtype == wtype) {
                        isAcceptableWeapon = true;
                        break;
                    }
                }
                if (!isAcceptableWeapon)
                    return false;
            }
            else {
                if (gInfoWeapon.wtype !== gInfoSkill.wtype)
                    return false;
            }
        }
        if (gInfoSkill.inventory) {
            for (const item of gInfoSkill.inventory) {
                if (!this.hasItem(item))
                    return false;
            }
        }
        if (gInfoSkill.slot) {
            for (const [slot, item] of gInfoSkill.slot) {
                if (!this.character.slots[slot])
                    return false;
                if (this.character.slots[slot].name !== item)
                    return false;
            }
        }
        if (gInfoSkill.class) {
            let compatibleClass = false;
            for (const c of gInfoSkill.class) {
                if (c == this.character.ctype) {
                    compatibleClass = true;
                    break;
                }
            }
            if (!compatibleClass)
                return false;
        }
        if (gInfoSkill.requirements) {
            for (const s in gInfoSkill.requirements) {
                const stat = s;
                if (this.character[stat] < gInfoSkill.requirements[stat])
                    return false;
            }
        }
        if (this.character.s.dampened) {
            if (skill == "blink")
                return false;
        }
        if (this.character.ctype == "merchant" && skill == "attack") {
            if (!this.character.slots.mainhand)
                return false;
            if (this.character.slots.mainhand.name !== "dartgun")
                return false;
            if (this.character.gold < 100)
                return false;
        }
        return true;
    }
    compound(item1Pos, item2Pos, item3Pos, cscrollPos, offeringPos) {
        const item1Info = this.character.items[item1Pos];
        const item2Info = this.character.items[item2Pos];
        const item3Info = this.character.items[item3Pos];
        const cscrollInfo = this.character.items[cscrollPos];
        if (!item1Info)
            return Promise.reject(`There is no item in inventory slot ${item1Pos} (item1).`);
        if (!item2Info)
            return Promise.reject(`There is no item in inventory slot ${item2Pos} (item2).`);
        if (!item3Info)
            return Promise.reject(`There is no item in inventory slot ${item3Pos} (item3).`);
        if (!cscrollInfo)
            return Promise.reject(`There is no item in inventory slot ${cscrollPos} (cscroll).`);
        if (item1Info.name != item2Info.name || item1Info.name != item3Info.name)
            return Promise.reject("You can only combine 3 of the same items.");
        if (item1Info.level != item2Info.level || item1Info.level != item3Info.level)
            return Promise.reject("You can only combine 3 items of the same level.");
        const compoundComplete = new Promise((resolve, reject) => {
            const completeCheck = (data) => {
                if (data.type == "compound") {
                    this.socket.removeListener("upgrade", completeCheck);
                    this.socket.removeListener("game_response", gameResponseCheck);
                    this.socket.removeListener("player", playerCheck);
                    resolve(data.success == 1);
                }
            };
            const playerCheck = (data) => {
                if (!data.hitchhikers)
                    return;
                for (const [event, datum] of data.hitchhikers) {
                    if (event == "game_response" && datum.response == "compound_fail") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        resolve(false);
                        return;
                    }
                    else if (event == "game_response" && datum.response == "compound_success") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        resolve(true);
                        return;
                    }
                }
            };
            const gameResponseCheck = (data) => {
                if (typeof data == "object") {
                    if (data.response == "bank_restrictions" && data.place == "compound") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        reject("You can't compound items in the bank.");
                    }
                }
                else if (typeof data == "string") {
                    if (data == "compound_no_item") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        reject();
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("upgrade", completeCheck);
                this.socket.removeListener("game_response", gameResponseCheck);
                this.socket.removeListener("player", playerCheck);
                reject("compound timeout (60000ms)");
            }, 60000);
            this.socket.on("upgrade", completeCheck);
            this.socket.on("game_response", gameResponseCheck);
            this.socket.on("player", playerCheck);
        });
        this.socket.emit("compound", {
            "items": [item1Pos, item2Pos, item3Pos],
            "scroll_num": cscrollPos,
            "clevel": item1Info.level
        });
        return compoundComplete;
    }
    craft(item) {
        const gInfo = this.G.craft[item];
        if (!gInfo)
            return Promise.reject(`Can not find a recipe for ${item}.`);
        if (gInfo.cost > this.character.gold)
            return Promise.reject(`We don't have enough gold to craft ${item}.`);
        const itemPositions = [];
        for (let i = 0; i < gInfo.items.length; i++) {
            const lookQ = gInfo.items[i][0];
            const lookName = gInfo.items[i][1];
            const lookLevel = gInfo.items[i][2];
            const searchArgs = {
                quantityGreaterThan: lookQ > 1 ? lookQ : undefined,
                level: lookLevel
            };
            const itemPos = this.locateItem(lookName, this.character.items, searchArgs);
            if (itemPos == undefined)
                return Promise.reject(`We don't have ${lookQ} ${lookName} to craft ${item}.`);
            itemPositions.push(itemPos);
        }
        this.socket.emit("craft", { items: itemPositions });
        return;
    }
    depositGold(gold) {
        if (this.character.map !== "bank")
            return Promise.reject("We need to be in 'bank' to deposit gold.");
        if (gold <= 0)
            return Promise.reject("We can't deposit 0 or less gold");
        if (gold > this.character.gold) {
            gold = this.character.gold;
            console.warn(`We are only going to deposit ${gold} gold.`);
        }
        this.socket.emit("bank", { operation: "deposit", amount: gold });
    }
    depositItem(inventoryPos, bankPack, bankSlot = -1) {
        if (this.character.map !== "bank" && this.character.map !== "bank_b" && this.character.map !== "bank_u")
            return Promise.reject(`We're not in the bank (we're in '${this.character.map}')`);
        const item = this.character.items[inventoryPos];
        if (!item)
            return Promise.reject(`There is no item in inventory slot ${inventoryPos}.`);
        if (bankPack) {
            const bankPackNum = Number.parseInt(bankPack.substr(5, 2));
            if ((this.character.map == "bank" && bankPackNum < 0 && bankPackNum > 7)
                || (this.character.map == "bank_b" && bankPackNum < 8 && bankPackNum > 23)
                || (this.character.map == "bank_u" && bankPackNum < 24 && bankPackNum > 47)) {
                return Promise.reject(`We can not access ${bankPack} on ${this.character.map}.`);
            }
        }
        else {
            bankSlot = undefined;
            let packFrom;
            let packTo;
            if (this.character.map == "bank") {
                packFrom = 0;
                packTo = 7;
            }
            else if (this.character.map == "bank_b") {
                packFrom = 8;
                packTo = 23;
            }
            else if (this.character.map == "bank_u") {
                packFrom = 24;
                packTo = 47;
            }
            const numStackable = this.G.items[item.name].s;
            let emptyPack;
            let emptySlot;
            for (let packNum = packFrom; packNum <= packTo; packNum++) {
                const packName = `items${packNum}`;
                const pack = this.bank[packName];
                if (!pack)
                    continue;
                for (let slotNum = 0; slotNum < pack.length; slotNum++) {
                    const slot = pack[slotNum];
                    if (!slot) {
                        if (!numStackable) {
                            bankPack = packName;
                            bankSlot = slotNum;
                            break;
                        }
                        else if (!emptyPack && emptySlot == undefined) {
                            emptyPack = packName;
                            emptySlot = slotNum;
                        }
                    }
                    else if (numStackable && slot.name == item.name && (slot.q + item.q <= numStackable)) {
                        bankPack = packName;
                        bankSlot = -1;
                        break;
                    }
                }
                if (bankPack && bankSlot !== undefined)
                    break;
            }
            if (bankPack == undefined && bankSlot == undefined && emptyPack !== undefined && emptySlot !== undefined) {
                bankPack = emptyPack;
                bankSlot = emptySlot;
            }
            else if (bankPack === undefined && bankSlot === undefined && emptyPack === undefined && emptySlot === undefined) {
                return Promise.reject(`Bank is full. There is nowhere to place '${item.name}'.`);
            }
        }
        const bankItemCount = this.countItem(item.name, this.bank[bankPack]);
        const swapped = new Promise((resolve, reject) => {
            const checkDeposit = (data) => {
                if (!data.user) {
                    if (data.map !== "bank" && data.map !== "bank_b" && data.map !== "bank_u") {
                        this.socket.removeListener("player", checkDeposit);
                        return reject(`We're not in the bank (we're in '${data.map}')`);
                    }
                }
                else {
                    const newBankItemCount = this.countItem(item.name, data.user[bankPack]);
                    if ((item.q && newBankItemCount == (bankItemCount + item.q))
                        || (!item.q && newBankItemCount == (bankItemCount + 1))) {
                        this.socket.removeListener("player", checkDeposit);
                        return resolve();
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkDeposit);
                reject(`depositItem timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkDeposit);
        });
        this.socket.emit("bank", { operation: "swap", pack: bankPack, str: bankSlot, inv: inventoryPos });
        return swapped;
    }
    equip(inventoryPos, equipSlot) {
        if (!this.character.items[inventoryPos])
            return Promise.reject(`No item in inventory slot ${inventoryPos}.`);
        const iInfo = this.character.items[inventoryPos];
        const equipFinished = new Promise((resolve, reject) => {
            const equipCheck = (data) => {
                if (equipSlot) {
                    const item = data.slots[equipSlot];
                    if (item
                        && item.name == iInfo.name
                        && item.level == iInfo.level
                        && item.p == iInfo.p) {
                        this.socket.removeListener("player", equipCheck);
                        this.socket.removeListener("disappearing_text", cantEquipCheck);
                        resolve();
                    }
                }
                else {
                    for (const slot in data.slots) {
                        const item = data.slots[slot];
                        if (item && item.name == iInfo.name) {
                            this.socket.removeListener("player", equipCheck);
                            this.socket.removeListener("disappearing_text", cantEquipCheck);
                            resolve();
                        }
                    }
                }
            };
            const cantEquipCheck = (data) => {
                if (data.id == this.character.id && data.message == "CAN'T EQUIP") {
                    this.socket.removeListener("player", equipCheck);
                    this.socket.removeListener("disappearing_text", cantEquipCheck);
                    reject(`Can't equip '${inventoryPos}' (${iInfo.name})`);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", equipCheck);
                this.socket.removeListener("disappearing_text", cantEquipCheck);
                reject(`equip timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", equipCheck);
            this.socket.on("disappearing_text", cantEquipCheck);
        });
        this.socket.emit("equip", { num: inventoryPos, slot: equipSlot });
        return equipFinished;
    }
    exchange(inventoryPos) {
        if (!this.character.items[inventoryPos])
            return Promise.reject(`No item in inventory slot ${inventoryPos}.`);
        const exchangeFinished = new Promise((resolve, reject) => {
            const completeCheck = (data) => {
                if (data.type == "exchange") {
                    this.socket.removeListener("upgrade", completeCheck);
                    this.socket.removeListener("game_response", bankCheck);
                    resolve(data.success == 1);
                }
            };
            const bankCheck = (data) => {
                if (typeof data == "object" && data.response == "bank_restrictions" && data.place == "upgrade") {
                    this.socket.removeListener("upgrade", completeCheck);
                    this.socket.removeListener("game_response", bankCheck);
                    reject("You can't exchange items in the bank.");
                }
                else if (typeof data == "string") {
                    if (data == "exchange_notenough") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", bankCheck);
                        reject("We don't have enough items to exchange.");
                    }
                    else if (data == "exchange_existing") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", bankCheck);
                        reject("We are already exchanging something.");
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("upgrade", completeCheck);
                this.socket.removeListener("game_response", bankCheck);
                reject(`exchange timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("game_response", bankCheck);
            this.socket.on("upgrade", completeCheck);
        });
        this.socket.emit("exchange", { item_num: inventoryPos, q: this.character.items[inventoryPos]?.q });
        return exchangeFinished;
    }
    finishMonsterHuntQuest() {
        this.socket.emit("monsterhunt");
    }
    getMonsterHuntQuest() {
        if (this.character.ctype == "merchant")
            return Promise.reject("Merchants can't do Monster Hunts.");
        let close = false;
        for (const npc of this.G.maps[this.character.map].npcs) {
            if (npc.id !== "monsterhunter")
                continue;
            if (Tools_1.Tools.distance(this.character, { x: npc.position[0], y: npc.position[1] }) <= Constants_1.Constants.NPC_INTERACTION_DISTANCE) {
                close = true;
                break;
            }
        }
        if (!close)
            return Promise.reject("We are too far away from the Monster Hunter NPC.");
        if (this.character.s.monsterhunt && this.character.s.monsterhunt.c > 0)
            return Promise.reject(`We can't get a new monsterhunt. We have ${this.character.s.monsterhunt.ms}ms left to kill ${this.character.s.monsterhunt.c} ${this.character.s.monsterhunt.id}s.`);
        if (this.character.s.monsterhunt && this.character.s.monsterhunt.c == 0) {
            console.warn("We are going to complete the current monster quest first");
            this.finishMonsterHuntQuest();
        }
        const questGot = new Promise((resolve, reject) => {
            const failCheck = (data) => {
                if (data == "ecu_get_closer") {
                    this.socket.removeListener("game_response", failCheck);
                    this.socket.removeListener("player", successCheck);
                    reject("Too far away from Monster Hunt NPC.");
                }
                else if (data == "monsterhunt_merchant") {
                    this.socket.removeListener("game_response", failCheck);
                    this.socket.removeListener("player", successCheck);
                    reject("Merchants can't do Monster Hunts.");
                }
            };
            const successCheck = (data) => {
                if (!data.hitchhikers)
                    return;
                for (const hitchhiker of data.hitchhikers) {
                    if (hitchhiker[0] == "game_response" && hitchhiker[1] == "monsterhunt_started") {
                        this.socket.removeListener("game_response", failCheck);
                        this.socket.removeListener("player", successCheck);
                        resolve();
                        return;
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("game_response", failCheck);
                this.socket.removeListener("player", successCheck);
                reject(`getMonsterHuntQuest timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("game_response", failCheck);
            this.socket.on("player", successCheck);
        });
        this.socket.emit("monsterhunt");
        return questGot;
    }
    getPontyItems() {
        const pontyItems = new Promise((resolve, reject) => {
            const distanceCheck = (data) => {
                if (data == "buy_get_closer") {
                    this.socket.removeListener("game_response", distanceCheck);
                    this.socket.removeListener("secondhands", secondhandsItems);
                    reject("Too far away from secondhands NPC.");
                }
            };
            const secondhandsItems = (data) => {
                this.socket.removeListener("game_response", distanceCheck);
                this.socket.removeListener("secondhands", secondhandsItems);
                resolve(data);
            };
            setTimeout(() => {
                this.socket.removeListener("game_response", distanceCheck);
                this.socket.removeListener("secondhands", secondhandsItems);
                reject(`getPontyItems timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("secondhands", secondhandsItems);
            this.socket.on("game_response", distanceCheck);
        });
        this.socket.emit("secondhands");
        return pontyItems;
    }
    isFull() {
        return this.character.esize >= this.character.isize;
    }
    leaveMap() {
        const leaveComplete = new Promise((resolve, reject) => {
            this.socket.once("new_map", (data) => {
                if (data.name == "main")
                    resolve();
                else
                    reject(`We are now in ${data.name}, but we should be in main`);
            });
            setTimeout(() => {
                reject(`leaveMap timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
        });
        this.socket.emit("leave");
        return leaveComplete;
    }
    leaveParty() {
        this.socket.emit("party", { event: "leave" });
    }
    async move(x, y, safetyCheck = true) {
        if (this.character.x == x && this.character.y == y)
            return Promise.resolve({ map: this.character.map, y: this.character.y, x: this.character.x });
        let to = { map: this.character.map, x: x, y: y };
        if (safetyCheck) {
            to = index_1.Pathfinder.getSafeWalkTo({ map: this.character.map, x: this.character.x, y: this.character.y }, { map: this.character.map, x, y });
            if (to.x !== x || to.y !== y) {
                console.warn(`move: We can't move to {x: ${x}, y: ${y}} safely. We will move to {x: ${to.x}, y: ${to.y}}.`);
            }
        }
        const moveFinished = new Promise((resolve, reject) => {
            let timeToFinishMove = 1 + Tools_1.Tools.distance(this.character, { x: to.x, y: to.y }) / this.character.speed;
            const checkPlayer = async (data) => {
                if (!data.moving || data.going_x != to.x || data.going_y != to.y) {
                    const newData = await this.requestPlayerData();
                    if (!newData.moving || newData.going_x != to.x || newData.going_y != to.y) {
                        clearTimeout(timeout);
                        this.socket.removeListener("player", checkPlayer);
                        reject(`move to ${to.x}, ${to.y} failed`);
                    }
                }
                else {
                    timeToFinishMove = Tools_1.Tools.distance(this.character, { x: data.x, y: data.y }) / data.speed;
                    clearTimeout(timeout);
                    timeout = setTimeout(checkPosition, timeToFinishMove);
                }
            };
            const checkPosition = () => {
                this.updatePositions();
                timeToFinishMove = 1 + Tools_1.Tools.distance(this.character, { x: to.x, y: to.y }) / this.character.speed;
                if (this.character.x == to.x && this.character.y == to.y) {
                    this.socket.removeListener("player", checkPlayer);
                    resolve(this.character);
                }
                else if (this.character.moving && this.character.going_x == to.x && this.character.going_y == to.y) {
                    timeout = setTimeout(checkPosition, timeToFinishMove);
                }
                else {
                    this.socket.removeListener("player", checkPlayer);
                    reject(`move to ${to.x}, ${to.y} failed (we're currently going to ${this.character.going_x}, ${this.character.going_y})`);
                }
            };
            let timeout = setTimeout(checkPosition, timeToFinishMove);
            this.socket.on("player", checkPlayer);
        });
        this.socket.emit("move", {
            x: this.character.x,
            y: this.character.y,
            going_x: to.x,
            going_y: to.y,
            m: this.character.m
        });
        this.updatePositions();
        this.character.going_x = to.x;
        this.character.going_y = to.y;
        this.character.moving = true;
        return moveFinished;
    }
    openChest(id) {
        const chestOpened = new Promise((resolve, reject) => {
            const openCheck = (data) => {
                if (data.id == id) {
                    this.socket.removeListener("chest_opened", openCheck);
                    resolve(data);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("chest_opened", openCheck);
                reject(`openChest timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("chest_opened", openCheck);
        });
        this.socket.emit("open_chest", { id: id });
        return chestOpened;
    }
    regenHP() {
        const regenReceived = new Promise((resolve, reject) => {
            const regenCheck = (data) => {
                if (data.code && data.code.includes("pot_timeout")) {
                    this.socket.removeListener("eval", regenCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", regenCheck);
                reject(`regenHP timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", regenCheck);
        });
        this.socket.emit("use", { item: "hp" });
        return regenReceived;
    }
    regenMP() {
        const regenReceived = new Promise((resolve, reject) => {
            const regenCheck = (data) => {
                if (data.code && data.code.includes("pot_timeout")) {
                    this.socket.removeListener("eval", regenCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", regenCheck);
                reject(`regenMP timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", regenCheck);
        });
        this.socket.emit("use", { item: "mp" });
        return regenReceived;
    }
    respawn() {
        this.socket.emit("respawn");
    }
    scare() {
        const scared = new Promise((resolve, reject) => {
            let ids;
            const idsCheck = (data) => {
                if (data.type == "scare") {
                    ids = data.ids;
                    this.socket.removeListener("ui", idsCheck);
                }
            };
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]scare['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("ui", idsCheck);
                    this.socket.removeListener("eval", cooldownCheck);
                    resolve(ids);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("ui", idsCheck);
                this.socket.removeListener("eval", cooldownCheck);
                reject(`scare timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("ui", idsCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { name: "scare" });
        return scared;
    }
    async sell(itemPos, quantity = 1) {
        this.socket.emit("sell", { num: itemPos, quantity: quantity });
    }
    async sendCM(to, message) {
        this.socket.emit("cm", { to: to, message: JSON.stringify(message) });
    }
    async sendGold(to, amount) {
        if (this.character.gold == 0)
            return Promise.reject("We have no gold to send.");
        if (!this.players.has(to))
            return Promise.reject(`We can not see ${to} to send gold.`);
        const goldSent = new Promise((resolve, reject) => {
            const sentCheck = (data) => {
                if (data == "trade_get_closer") {
                    this.socket.removeListener("game_response", sentCheck);
                    reject(`We are too far away from ${to} to send gold.`);
                }
                else if (typeof data == "object" && data.response == "gold_sent" && data.name == to) {
                    if (data.gold !== amount)
                        console.warn(`We wanted to send ${to} ${amount} gold, but we sent ${data.gold}.`);
                    this.socket.removeListener("game_response", sentCheck);
                    resolve(data.gold);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("game_response", sentCheck);
                reject(`sendGold timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("game_response", sentCheck);
        });
        this.socket.emit("send", { name: to, gold: amount });
        return goldSent;
    }
    sendItem(to, inventoryPos, quantity = 1) {
        if (!this.players.has(to))
            return Promise.reject(`"${to}" is not nearby.`);
        if (!this.character.items[inventoryPos])
            return Promise.reject(`No item in inventory slot ${inventoryPos}.`);
        if (this.character.items[inventoryPos]?.q < quantity)
            return Promise.reject(`We only have a quantity of ${this.character.items[inventoryPos].q}, not ${quantity}.`);
        const item = this.character.items[inventoryPos];
        const itemSent = new Promise((resolve, reject) => {
            const sentCheck = (data) => {
                if (data == "trade_get_closer") {
                    this.socket.removeListener("game_response", sentCheck);
                    reject(`sendItem failed, ${to} is too far away`);
                }
                else if (data == "send_no_space") {
                    this.socket.removeListener("game_response", sentCheck);
                    reject(`sendItem failed, ${to} has no inventory space`);
                }
                else if (typeof data == "object" && data.response == "item_sent" && data.name == to && data.item == item.name && data.q == quantity) {
                    this.socket.removeListener("game_response", sentCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("game_response", sentCheck);
                reject(`sendItem timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("game_response", sentCheck);
        });
        this.socket.emit("send", { name: to, num: inventoryPos, q: quantity });
        return itemSent;
    }
    sendPartyInvite(id) {
        this.socket.emit("party", { event: "invite", name: id });
    }
    sendPartyRequest(id) {
        this.socket.emit("party", { event: "request", name: id });
    }
    async smartMove(to, options = {
        getWithin: 0,
        useBlink: false
    }) {
        const started = Date.now();
        this.lastSmartMove = started;
        let fixedTo;
        let path;
        if (typeof to == "string") {
            for (const mapName in this.G.maps) {
                if (to !== mapName)
                    continue;
                const mainSpawn = this.G.maps[to].spawns[0];
                fixedTo = { map: to, x: mainSpawn[0], y: mainSpawn[1] };
                break;
            }
            if (!fixedTo) {
                for (const mtype in this.G.monsters) {
                    if (to !== mtype)
                        continue;
                    const locations = this.locateMonsters(mtype);
                    let closestDistance = Number.MAX_VALUE;
                    for (const location of locations) {
                        const potentialPath = await index_1.Pathfinder.getPath(this.character, location);
                        const distance = index_1.Pathfinder.computePathCost(potentialPath);
                        if (distance < closestDistance) {
                            path = potentialPath;
                            fixedTo = path[path.length - 1];
                            closestDistance = distance;
                        }
                    }
                    break;
                }
            }
            if (!fixedTo) {
                for (const mapName in this.G.maps) {
                    if (this.G.maps[mapName].ignore)
                        continue;
                    for (const npc of this.G.maps[mapName].npcs) {
                        if (to !== npc.id)
                            continue;
                        const locations = this.locateNPCs(npc.id);
                        let closestDistance = Number.MAX_VALUE;
                        for (const location of locations) {
                            const potentialPath = await index_1.Pathfinder.getPath(this.character, location);
                            const distance = index_1.Pathfinder.computePathCost(potentialPath);
                            if (distance < closestDistance) {
                                path = potentialPath;
                                fixedTo = path[path.length - 1];
                                closestDistance = distance;
                            }
                        }
                        break;
                    }
                }
            }
            if (!fixedTo)
                return Promise.reject(`Could not find a suitable destination for '${to}'`);
        }
        else if (to.x !== undefined && to.y !== undefined) {
            if (to.map)
                fixedTo = to;
            else
                fixedTo = { map: this.character.map, x: to.x, y: to.y };
        }
        else {
            console.log(to);
            return Promise.reject("'to' is unsuitable for smartMove. We need a 'map', an 'x', and a 'y'.");
        }
        if (options && options.getWithin !== undefined && Tools_1.Tools.distance(this.character, fixedTo) <= options.getWithin)
            return Promise.resolve(this.character);
        if (!path)
            path = await index_1.Pathfinder.getPath(this.character, fixedTo);
        let lastMove = -1;
        for (let i = 0; i < path.length; i++) {
            let currentMove = path[i];
            if (started < this.lastSmartMove) {
                if (typeof to == "string")
                    return Promise.reject(`smartMove to ${to} cancelled (new smartMove started)`);
                else
                    return Promise.reject(`smartMove to ${to.map}:${to.x},${to.y} cancelled (new smartMove started)`);
            }
            if (Tools_1.Tools.distance(this.character, fixedTo) < options.getWithin) {
                break;
            }
            if (currentMove.type == "move" && this.character.map == fixedTo.map && options.getWithin > 0) {
                const angle = Math.atan2(this.character.y - fixedTo.y, this.character.x - fixedTo.x);
                const potentialMove = {
                    type: "move",
                    map: this.character.map,
                    x: fixedTo.x + Math.cos(angle) * options.getWithin,
                    y: fixedTo.y + Math.sin(angle) * options.getWithin
                };
                if (index_1.Pathfinder.canWalk(this.character, potentialMove)) {
                    i = path.length;
                    currentMove = potentialMove;
                }
            }
            if (currentMove.type == "move") {
                for (let j = i + 1; j < path.length; j++) {
                    const potentialMove = path[j];
                    if (potentialMove.map !== currentMove.map)
                        break;
                    if (potentialMove.type == "town")
                        break;
                    if (potentialMove.type == "move" && index_1.Pathfinder.canWalk(this.character, potentialMove)) {
                        console.log("skip check success!");
                        i = j;
                        currentMove = potentialMove;
                    }
                }
            }
            if (options.useBlink && this.canUse("blink")) {
                let blinked = false;
                for (let j = path.length - 1; j > i; j--) {
                    const potentialMove = path[j];
                    if (potentialMove.map == currentMove.map) {
                        await this.blink(potentialMove.x, potentialMove.y);
                        i = j;
                        blinked = true;
                        break;
                    }
                }
                if (blinked)
                    continue;
            }
            try {
                if (currentMove.type == "leave") {
                    await this.leaveMap();
                }
                else if (currentMove.type == "move") {
                    if (currentMove.map !== this.character.map) {
                        return Promise.reject(`We are supposed to be in ${currentMove.map}, but we are in ${this.character.map}`);
                    }
                    await this.move(currentMove.x, currentMove.y, false);
                }
                else if (currentMove.type == "town") {
                    await this.warpToTown();
                }
                else if (currentMove.type == "transport") {
                    await this.transport(currentMove.map, currentMove.spawn);
                }
            }
            catch (e) {
                console.error(e);
                await this.requestPlayerData();
                if (lastMove == i)
                    return Promise.reject("We are having some trouble smartMoving...");
                lastMove = i;
                i--;
            }
        }
        return { map: this.character.map, x: this.character.x, y: this.character.y };
    }
    async stopSmartMove() {
        this.lastSmartMove = Date.now();
        return this.move(this.character.x, this.character.y);
    }
    stopWarpToTown() {
        this.socket.emit("stop", { action: "town" });
    }
    throwSnowball(target) {
        if (this.G.skills.snowball.mp > this.character.mp)
            return Promise.reject("Not enough MP to throw snowball");
        const throwStarted = new Promise((resolve, reject) => {
            let projectile;
            const attackCheck = (data) => {
                if (data.attacker == this.character.id
                    && data.type == "snowball"
                    && data.target == target) {
                    projectile = data.pid;
                }
            };
            const cooldownCheck = (data) => {
                if (/skill_timeout\s*\(\s*['"]snowball['"]\s*,?\s*(\d+\.?\d+?)?\s*\)/.test(data.code)) {
                    this.socket.removeListener("action", attackCheck);
                    this.socket.removeListener("eval", cooldownCheck);
                    resolve(projectile);
                }
            };
            setTimeout(() => {
                this.socket.removeListener("action", attackCheck);
                this.socket.removeListener("eval", cooldownCheck);
                reject(`throwsnowball timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("action", attackCheck);
            this.socket.on("eval", cooldownCheck);
        });
        this.socket.emit("skill", { name: "snowball", id: target });
        return throwStarted;
    }
    transport(map, spawn) {
        const transportComplete = new Promise((resolve, reject) => {
            this.socket.once("new_map", (data) => {
                if (data.name == map)
                    resolve();
                else
                    reject(`We are now in ${data.name}, but we should be in ${map}`);
            });
            setTimeout(() => {
                reject(`transport timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
        });
        this.socket.emit("transport", { to: map, s: spawn });
        return transportComplete;
    }
    unequip(slot) {
        if (this.character.slots[slot] === null)
            return Promise.reject(`Slot ${slot} is empty; nothing to unequip.`);
        if (this.character.slots[slot] === undefined)
            return Promise.reject(`Slot ${slot} does not exist.`);
        const unequipped = new Promise((resolve, reject) => {
            const unequipCheck = (data) => {
                if (data.slots[slot] === null) {
                    this.socket.removeListener("player", unequipCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", unequipCheck);
                reject(`unequip timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
        });
        this.socket.emit("unequip", { slot: slot });
        return unequipped;
    }
    upgrade(itemPos, scrollPos) {
        if (this.character.map.startsWith("bank"))
            return Promise.reject("We can't upgrade things in the bank.");
        const itemInfo = this.character.items[itemPos];
        const scrollInfo = this.character.items[scrollPos];
        if (!itemInfo)
            return Promise.reject(`There is no item in inventory slot ${itemPos}.`);
        if (this.G.items[itemInfo.name].upgrade == undefined)
            return Promise.reject("This item is not upgradable.");
        if (!scrollInfo)
            return Promise.reject(`There is no scroll in inventory slot ${scrollPos}.`);
        const upgradeComplete = new Promise((resolve, reject) => {
            const completeCheck = (data) => {
                if (data.type == "upgrade") {
                    this.socket.removeListener("upgrade", completeCheck);
                    this.socket.removeListener("game_response", gameResponseCheck);
                    this.socket.removeListener("player", playerCheck);
                    resolve(data.success == 1);
                }
            };
            const playerCheck = (data) => {
                if (!data.hitchhikers)
                    return;
                for (const [event, datum] of data.hitchhikers) {
                    if (event == "game_response" && datum.response == "upgrade_fail" && datum.num == itemPos) {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        resolve(false);
                        return;
                    }
                    else if (event == "game_response" && datum.response == "upgrade_success" && datum.num == itemPos) {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        resolve(true);
                        return;
                    }
                }
            };
            const gameResponseCheck = (data) => {
                if (typeof data == "object" && data.response == "bank_restrictions" && data.place == "upgrade") {
                    this.socket.removeListener("upgrade", completeCheck);
                    this.socket.removeListener("game_response", gameResponseCheck);
                    this.socket.removeListener("player", playerCheck);
                    reject("You can't upgrade items in the bank.");
                }
                else if (typeof data == "string") {
                    if (data == "bank_restrictions") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        reject("We can't upgrade things in the bank.");
                    }
                    else if (data == "upgrade_in_progress") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        reject("We are already upgrading something.");
                    }
                    else if (data == "upgrade_incompatible_scroll") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        reject(`The scroll we're trying to use (${scrollInfo.name}) isn't a high enough grade to upgrade this item.`);
                    }
                    else if (data == "upgrade_success") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        resolve(true);
                    }
                    else if (data == "upgrade_fail") {
                        this.socket.removeListener("upgrade", completeCheck);
                        this.socket.removeListener("game_response", gameResponseCheck);
                        this.socket.removeListener("player", playerCheck);
                        resolve(false);
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("upgrade", completeCheck);
                this.socket.removeListener("game_response", gameResponseCheck);
                this.socket.removeListener("player", playerCheck);
                reject("upgrade timeout (60000ms)");
            }, 60000);
            this.socket.on("upgrade", completeCheck);
            this.socket.on("game_response", gameResponseCheck);
            this.socket.on("player", playerCheck);
        });
        this.socket.emit("upgrade", { item_num: itemPos, scroll_num: scrollPos, clevel: this.character.items[itemPos].level });
        return upgradeComplete;
    }
    useHPPot(itemPos) {
        if (!this.character.items[itemPos])
            return Promise.reject(`There is no item in inventory slot ${itemPos}.`);
        const healReceived = new Promise((resolve, reject) => {
            const healCheck = (data) => {
                if (data.code && data.code.includes("pot_timeout")) {
                    this.socket.removeListener("eval", healCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", healCheck);
                reject(`useHPPot timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", healCheck);
        });
        this.socket.emit("equip", { num: itemPos });
        return healReceived;
    }
    useMPPot(itemPos) {
        if (!this.character.items[itemPos])
            return Promise.reject(`There is no item in inventory slot ${itemPos}.`);
        const healReceived = new Promise((resolve, reject) => {
            const healCheck = (data) => {
                if (data.code && data.code.includes("pot_timeout")) {
                    this.socket.removeListener("eval", healCheck);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("eval", healCheck);
                reject(`useMPPot timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("eval", healCheck);
        });
        this.socket.emit("equip", { num: itemPos });
        return healReceived;
    }
    warpToJail() {
        return this.move(Number.MAX_VALUE, Number.MAX_VALUE, false);
    }
    warpToTown() {
        let startedWarp = false;
        const warpComplete = new Promise((resolve, reject) => {
            const failCheck = (data) => {
                if (!startedWarp && data.c.town && data.c.town.ms == 3000) {
                    startedWarp = true;
                    return;
                }
                if (startedWarp && !data.c.town) {
                    this.socket.removeListener("player", failCheck);
                    this.socket.removeListener("new_map", warpedCheck2);
                    reject("warpToTown failed.");
                }
            };
            const warpedCheck2 = (data) => {
                if (data.effect == 1) {
                    this.socket.removeListener("player", failCheck);
                    this.socket.removeListener("new_map", warpedCheck2);
                    resolve({ map: data.name, x: data.x, y: data.y });
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", failCheck);
                this.socket.removeListener("new_map", warpedCheck2);
                reject("warpToTown timeout (5000ms)");
            }, 5000);
            this.socket.on("new_map", warpedCheck2);
            this.socket.on("player", failCheck);
        });
        this.socket.emit("town");
        return warpComplete;
    }
    withdrawGold(gold) {
        if (this.character.map !== "bank")
            return Promise.reject("We need to be in 'bank' to withdraw gold.");
        if (gold <= 0)
            return Promise.reject("We can't withdraw 0 or less gold.");
        if (gold > this.bank.gold) {
            gold = this.bank.gold;
            console.warn(`We are only going to withdraw ${gold} gold.`);
        }
        this.socket.emit("bank", { operation: "withdraw", amount: gold });
    }
    withdrawItem(bankPack, bankPos, inventoryPos = -1) {
        const item = this.bank[bankPack][bankPos];
        if (!item)
            return Promise.reject(`There is no item in bank ${bankPack}[${bankPos}]`);
        const bankPackNum = Number.parseInt(bankPack.substr(5, 2));
        if ((this.character.map == "bank" && bankPackNum < 0 && bankPackNum > 7)
            || (this.character.map == "bank_b" && bankPackNum < 8 && bankPackNum > 23)
            || (this.character.map == "bank_u" && bankPackNum < 24 && bankPackNum > 47)) {
            return Promise.reject(`We can not access ${bankPack} on ${this.character.map}.`);
        }
        const itemCount = this.countItem(item.name);
        const swapped = new Promise((resolve, reject) => {
            const checkWithdrawal = (data) => {
                const newCount = this.countItem(item.name, data.items);
                if (item.q && newCount == (itemCount + item.q)
                    || !item.q && newCount == (itemCount + 1)) {
                    this.socket.removeListener("player", checkWithdrawal);
                    return resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkWithdrawal);
                reject(`withdrawItem timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkWithdrawal);
        });
        console.log({ operation: "swap", pack: bankPack, str: bankPos, inv: inventoryPos });
        this.socket.emit("bank", { operation: "swap", pack: bankPack, str: bankPos, inv: inventoryPos });
        return swapped;
    }
    countItem(item, inventory = this.character.items, args) {
        let count = 0;
        for (const inventoryItem of inventory) {
            if (!inventoryItem)
                continue;
            if (inventoryItem.name !== item)
                continue;
            if (args) {
                if (args.levelGreaterThan !== undefined) {
                    if (inventoryItem.level == undefined)
                        continue;
                    if (inventoryItem.level <= args.levelGreaterThan)
                        continue;
                }
                if (args.levelLessThan !== undefined) {
                    if (inventoryItem.level == undefined)
                        continue;
                    if (inventoryItem.level >= args.levelLessThan)
                        continue;
                }
            }
            if (inventoryItem.q) {
                count += inventoryItem.q;
            }
            else {
                count += 1;
            }
        }
        return count;
    }
    getCooldown(skill) {
        if (this.G.skills[skill].share)
            skill = this.G.skills[skill].share;
        const nextSkill = this.nextSkill.get(skill);
        if (!nextSkill)
            return 0;
        const cooldown = nextSkill.getTime() - Date.now();
        if (cooldown < 0)
            return 0;
        return cooldown;
    }
    getNearestMonster(mtype) {
        let closest;
        let closestD = Number.MAX_VALUE;
        this.entities.forEach((entity) => {
            if (mtype && entity.type != mtype)
                return;
            const d = Tools_1.Tools.distance(this.character, entity);
            if (d < closestD) {
                closest = entity;
                closestD = d;
            }
        });
        if (closest)
            return { monster: closest, distance: closestD };
    }
    getNearestAttackablePlayer() {
        if (!this.isPVP())
            return undefined;
        let closest;
        let closestD = Number.MAX_VALUE;
        this.players.forEach((player) => {
            if (player.s?.invincible)
                return;
            if (player.npc)
                return;
            const d = Tools_1.Tools.distance(this.character, player);
            if (d < closestD) {
                closest = player;
                closestD = d;
            }
        });
        if (closest)
            return { player: closest, distance: closestD };
    }
    hasItem(iN, inv = this.character.items, args) {
        for (let i = 0; i < inv.length; i++) {
            const item = inv[i];
            if (!item)
                continue;
            if (args) {
                if (args.levelGreaterThan !== undefined) {
                    if (item.level == undefined)
                        continue;
                    if (item.level <= args.levelGreaterThan)
                        continue;
                }
                if (args.levelLessThan !== undefined) {
                    if (item.level == undefined)
                        continue;
                    if (item.level >= args.levelLessThan)
                        continue;
                }
            }
            if (item.name == iN)
                return true;
        }
        return false;
    }
    isEquipped(itemName) {
        for (const slot in this.character.slots) {
            if (!this.character.slots[slot])
                continue;
            if (this.character.slots[slot].name == itemName)
                return true;
        }
        return false;
    }
    isPVP() {
        if (this.G[this.character.map].pvp)
            return true;
        return this.server.pvp;
    }
    locateDuplicateItems(inventory = this.character.items) {
        const items = [];
        for (let i = 0; i < inventory.length; i++) {
            const item = inventory[i];
            if (!item)
                continue;
            items.push({ ...item, slotNum: i });
        }
        items.sort((a, b) => {
            const n = a.name.localeCompare(b.name);
            if (n !== 0)
                return n;
            if (a.level !== undefined && b.level !== undefined && a.level !== b.level)
                return a.level - b.level;
            if (a.p !== undefined && b.p === undefined)
                return -1;
            return 0;
        });
        const duplicates = {};
        for (let i = 0; i < items.length - 1; i++) {
            const item1 = items[i];
            for (let j = i + 1; j < items.length; j++) {
                const item2 = items[j];
                if (item1.name === item2.name) {
                    if (j == i + 1) {
                        duplicates[item1.name] = [item1.slotNum, item2.slotNum];
                    }
                    else {
                        duplicates[item1.name].push(item2.slotNum);
                    }
                }
                else {
                    i = j - 1;
                    break;
                }
            }
        }
        return duplicates;
    }
    locateItem(iN, inv = this.character.items, args) {
        for (let i = 0; i < inv.length; i++) {
            const item = inv[i];
            if (!item)
                continue;
            if (item.name !== iN)
                continue;
            if (args) {
                if (args.level !== undefined) {
                    if (item.level !== args.level)
                        continue;
                }
                if (args.levelGreaterThan !== undefined) {
                    if (item.level == undefined)
                        continue;
                    if (item.level <= args.levelGreaterThan)
                        continue;
                }
                if (args.levelLessThan !== undefined) {
                    if (item.level == undefined)
                        continue;
                    if (item.level >= args.levelLessThan)
                        continue;
                }
                if (args.quantityGreaterThan !== undefined) {
                    if (item.q == undefined)
                        continue;
                    if (item.q <= args.quantityGreaterThan)
                        continue;
                }
            }
            return i;
        }
        return undefined;
    }
    locateItems(itemName, inventory = this.character.items, filters) {
        const found = [];
        for (let i = 0; i < inventory.length; i++) {
            const item = inventory[i];
            if (!item)
                continue;
            if (filters) {
                if (filters.levelGreaterThan) {
                    if (item.level == undefined)
                        continue;
                    if (item.level <= filters.levelGreaterThan)
                        continue;
                }
                if (filters.levelLessThan) {
                    if (item.level == undefined)
                        continue;
                    if (item.level >= filters.levelLessThan)
                        continue;
                }
            }
            if (item.name == itemName) {
                found.push(i);
            }
        }
        return found;
    }
    locateMonsters(mType) {
        const locations = [];
        if (mType == "goldenbat")
            mType = "bat";
        else if (mType == "snowman")
            mType = "arcticbee";
        for (const mapName in this.G.maps) {
            if (this.G.maps[mapName].ignore)
                continue;
            const map = this.G.maps[mapName];
            if (map.instance || !map.monsters || map.monsters.length == 0)
                continue;
            for (const monsterSpawn of map.monsters) {
                if (monsterSpawn.type != mType)
                    continue;
                if (monsterSpawn.boundary) {
                    locations.push({ "map": mapName, "x": (monsterSpawn.boundary[0] + monsterSpawn.boundary[2]) / 2, "y": (monsterSpawn.boundary[1] + monsterSpawn.boundary[3]) / 2 });
                }
                else if (monsterSpawn.boundaries) {
                    for (const boundary of monsterSpawn.boundaries) {
                        locations.push({ "map": boundary[0], "x": (boundary[1] + boundary[3]) / 2, "y": (boundary[2] + boundary[4]) / 2 });
                    }
                }
            }
        }
        return locations;
    }
    locateNPCs(npcType) {
        const locations = [];
        for (const mapName in this.G.maps) {
            const map = this.G.maps[mapName];
            if (map.ignore)
                continue;
            if (map.instance || !map.npcs || map.npcs.length == 0)
                continue;
            for (const npc of map.npcs) {
                if (npc.id !== npcType)
                    continue;
                if (npc.position) {
                    locations.push({ map: mapName, x: npc.position[0], y: npc.position[1] });
                }
                else if (npc.positions) {
                    for (const position of npc.positions) {
                        locations.push({ map: mapName, x: position[0], y: position[1] });
                    }
                }
            }
        }
        return locations;
    }
}
exports.Character = Character;
//# sourceMappingURL=Character.js.map