"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Entity = void 0;
class Entity {
    constructor(data, map, G) {
        this.G = G;
        this.abilities = G.monsters[data.type].abilities;
        this.level = 1;
        this.max_hp = G.monsters[data.type]["hp"];
        this.max_mp = G.monsters[data.type]["mp"];
        this.map = map;
        this["1hp"] = G.monsters[data.type]["1hp"];
        this.apiercing = G.monsters[data.type].apiercing;
        this.attack = G.monsters[data.type].attack;
        this.cooperative = G.monsters[data.type].cooperative;
        this.damage_type = G.monsters[data.type].damage_type;
        this.evasion = G.monsters[data.type].evasion;
        this.frequency = G.monsters[data.type].frequency;
        this.hp = G.monsters[data.type].hp;
        this.immune = G.monsters[data.type].immune;
        this.mp = G.monsters[data.type].mp;
        this.range = G.monsters[data.type].range;
        this.reflection = G.monsters[data.type].reflection;
        this.speed = G.monsters[data.type].speed;
        this.xp = G.monsters[data.type].xp;
        this.updateData(data);
    }
    updateData(data) {
        if (this.id !== undefined && this.id !== data.id)
            throw Error("The entity's ID does not match");
        for (const key in data)
            this[key] = data[key];
    }
    isAttackingPartyMember(player) {
        if (this.target === undefined)
            return false;
        if (this.isAttackingUs(player))
            return true;
        if (player.party && player.party.list && player.party.list.includes(this.target))
            return true;
        return false;
    }
    isAttackingUs(player) {
        return this.target === player.character.id;
    }
    isTauntable(by) {
        if (this.target === undefined)
            return true;
        if (this.isAttackingPartyMember(by))
            return true;
        return false;
    }
    willBurnToDeath() {
        if (this["1hp"])
            return false;
        if (this.abilities && this.abilities.self_healing)
            return false;
        if (this.s.burned) {
            const burnTime = Math.max(0, (this.s.burned.ms - 500)) / 1000;
            const burnDamage = burnTime * this.s.burned.intensity;
            if (burnDamage > this.hp)
                return true;
        }
        return false;
    }
    willDieToProjectiles(projectiles) {
        if (this["1hp"] || this.evasion || this.reflection)
            return false;
        let incomingProjectileDamage = 0;
        for (const projectile of projectiles.values()) {
            if (projectile.target == this.id)
                incomingProjectileDamage += projectile.damage * 0.9;
            if (incomingProjectileDamage > this.hp)
                return true;
        }
        return false;
    }
}
exports.Entity = Entity;
//# sourceMappingURL=Entity.js.map