"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Merchant = void 0;
const Constants_1 = require("./Constants");
const PingCompensatedCharacter_1 = require("./PingCompensatedCharacter");
class Merchant extends PingCompensatedCharacter_1.PingCompensatedCharacter {
    closeMerchantStand() {
        if (!this.character.stand)
            return Promise.resolve();
        const closed = new Promise((resolve, reject) => {
            const checkStand = (data) => {
                if (!data.stand) {
                    this.socket.removeListener("player", checkStand);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkStand);
                reject(`closeMerchantStand timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkStand);
        });
        this.socket.emit("merchant", { close: 1 });
        return closed;
    }
    listForSale(itemPos, tradeSlot, price, quantity = 1) {
        const itemInfo = this.character.items[itemPos];
        if (!itemInfo)
            return Promise.reject(`We do not have an item in slot ${itemPos}`);
        this.socket.emit("equip", {
            num: itemPos,
            q: quantity,
            slot: tradeSlot,
            price: price
        });
    }
    mluck(target) {
        if (target !== this.character.id) {
            const player = this.players.get(target);
            if (!player)
                return Promise.reject(`Could not find ${target} to mluck.`);
            if (player.npc)
                return Promise.reject(`${target} is an NPC. You can't mluck NPCs.`);
            if (player.s.mluck && player.s.mluck.strong && player.s.mluck.f !== this.character.id)
                return Promise.reject(`${target} has a strong mluck from ${player.s.mluck.f}.`);
        }
        const mlucked = new Promise((resolve, reject) => {
            const mluckCheck = (data) => {
                for (const player of data.players) {
                    if (player.id == target
                        && player.s.mluck
                        && player.s.mluck.f == this.character.id) {
                        this.socket.removeListener("entities", mluckCheck);
                        this.socket.removeListener("game_response", failCheck);
                        resolve();
                    }
                }
            };
            const failCheck = async (data) => {
                if (typeof data == "string") {
                    if (data == "skill_too_far") {
                        this.socket.removeListener("entities", mluckCheck);
                        this.socket.removeListener("game_response", failCheck);
                        await this.requestPlayerData();
                        reject(`We are too far from ${target} to mluck.`);
                    }
                    else if (data == "no_level") {
                        this.socket.removeListener("entities", mluckCheck);
                        this.socket.removeListener("game_response", failCheck);
                        reject("We aren't a high enough level to use mluck.");
                    }
                }
            };
            setTimeout(() => {
                this.socket.removeListener("entities", mluckCheck);
                this.socket.removeListener("game_response", failCheck);
                reject(`mluck timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("entities", mluckCheck);
            this.socket.on("game_response", failCheck);
        });
        this.socket.emit("skill", { name: "mluck", id: target });
        return mlucked;
    }
    openMerchantStand() {
        if (this.character.stand)
            return Promise.resolve();
        let stand = this.locateItem("stand0");
        if (stand === undefined) {
            stand = this.locateItem("computer");
            if (stand === undefined)
                return Promise.reject("Could not find merchant stand ('stand0') or computer in inventory.");
        }
        const opened = new Promise((resolve, reject) => {
            const checkStand = (data) => {
                if (data.stand) {
                    this.socket.removeListener("player", checkStand);
                    resolve();
                }
            };
            setTimeout(() => {
                this.socket.removeListener("player", checkStand);
                reject(`openMerchantStand timeout (${Constants_1.Constants.TIMEOUT}ms)`);
            }, Constants_1.Constants.TIMEOUT);
            this.socket.on("player", checkStand);
        });
        this.socket.emit("merchant", { num: stand });
        return opened;
    }
}
exports.Merchant = Merchant;
//# sourceMappingURL=Merchant.js.map