export declare type GData = {
    achievements: {
        [T in string]: any;
    };
    base_gold: {
        [T in MonsterName]?: {
            [T in MapName]?: number;
        };
    };
    classes: {
        [T in CharacterType]: {
            damage_type: DamageType;
            doublehand: {
                [T in WeaponType]?: {
                    [T in StatType]?: number;
                };
            };
            mainhand: {
                [T in WeaponType]?: {
                    [T in StatType]?: number;
                };
            };
            offhand: {
                [T in WeaponType]?: {
                    [T in StatType]?: number;
                };
            };
        };
    };
    conditions: {
        [T in ConditionName]: {
            bad: boolean;
            buff: boolean;
            duration: number;
        } & {
            [T in StatType]?: number;
        };
    };
    craft: {
        [T in ItemName]?: {
            items: [number, ItemName, number?][];
            cost: number;
            quest?: NPCRole;
        };
    };
    dismantle: {
        [T in ItemName]?: {
            cost: number;
            items: [number, ItemName][];
        };
    };
    items: {
        [T in ItemName]: GItem;
    };
    geometry: {
        [T in MapName]: {
            max_x: number;
            max_y: number;
            min_x: number;
            min_y: number;
            x_lines: [number, number, number][];
            y_lines: [number, number, number][];
        };
    };
    maps: {
        [T in MapName]: {
            doors: DoorInfo[];
            key: string;
            ignore?: boolean;
            instance?: boolean;
            irregular?: boolean;
            monsters: {
                count: number;
                boundary?: [number, number, number, number];
                boundaries?: [MapName, number, number, number, number][];
                type: MonsterName;
            }[];
            mount: boolean;
            no_bounds?: boolean;
            npcs: GMapsNPC[];
            pvp?: boolean;
            on_death: number;
            ref: {
                [id: string]: IPosition & {
                    map: MapName;
                    in: MapName;
                    id: string;
                };
            };
            spawns: [number, number, number?][];
        };
    };
    monsters: {
        [T in MonsterName]: GMonster;
    };
    npcs: {
        [T in NPCType]: {
            id: NPCType;
            items?: ItemName[];
            name: string;
            places?: {
                [T in MapName]?: number;
            };
            role: NPCRole;
        };
    };
    quests: {
        [T in string]: PositionReal & {
            id: NPCType;
        };
    };
    skills: {
        [T in SkillName]: {
            apiercing?: number;
            class?: CharacterType[];
            cooldown: number;
            cooldown_multiplier?: number;
            damage_multiplier?: number;
            hostile?: boolean;
            inventory?: ItemName[];
            level?: number;
            monster?: boolean;
            mp?: number;
            name: string;
            range?: number;
            range_bonus?: number;
            range_multiplier?: number;
            ratio?: number;
            requirements?: {
                [T in StatType]?: number;
            };
            share?: SkillName;
            slot?: [SlotType, ItemName][];
            target?: boolean;
            targets?: boolean;
            warning?: string;
            wtype?: WeaponType | WeaponType[];
        };
    };
};
export declare type GMapsNPC = {
    id: NPCType;
    name?: string;
    position: [number, number, number?];
    positions: [number, number, number?];
};
export declare type GMonster = {
    abilities?: {
        [T in SkillName]?: any;
    };
    apiercing?: number;
    attack: number;
    charge?: number;
    cooperative?: boolean;
    damage_type: DamageType;
    evasion?: number;
    frequency: number;
    global?: boolean;
    hp: number;
    immune?: boolean;
    lucrativeness?: number;
    mp: number;
    range: number;
    reflection?: number;
    speed: number;
    xp: number;
};
export declare type GItem = {
    buy?: boolean;
    compound?: {
        [T in StatType]?: number;
    };
    damage?: DamageType;
    e?: number;
    g: number;
    grades?: [number, number];
    name: string;
    id: ItemName;
    quest: string;
    s: number;
    upgrade?: {
        [T in StatType]?: number;
    };
    type: ItemType;
    wtype: WeaponType;
} & {
    [T in StatType]?: number;
};
export declare type CharacterEntity = Entity & {
    bank?: BankInfo;
    c: {
        town?: {
            ms: number;
        };
    };
    ctype: CharacterType;
    items: ItemInfo[];
    gold: number;
    goldm: number;
    luckm: number;
    ping: number;
    q: {
        compound?: {};
        upgrade?: {
            ms: number;
            len: number;
            num: number;
        };
        exchange?: {};
    };
    xrange: number;
};
export declare type Entity = PositionMovable & {
    "1hp": number;
    aggro: number;
    apiercing: number;
    armor: number;
    attack: number;
    base: {
        "h": number;
        "v": number;
        "vn": number;
    };
    cooperative: boolean;
    ctype: CharacterType | NPCType;
    damage_type?: DamageType;
    evasion: number;
    frequency: number;
    humanoid?: boolean;
    hp: number;
    id: string;
    immune: boolean;
    last_ms: Date;
    level: number;
    max_hp: number;
    max_mp: number;
    moving: boolean;
    mp: number;
    mp_cost: number;
    mtype?: MonsterName;
    name: string;
    npc?: string;
    range: number;
    real_x: number;
    real_y: number;
    resistance: number;
    rip?: boolean;
    rpiercing: number;
    s: StatusInfo;
    slots: SlotInfo;
    speed: number;
    standed?: any;
    stoned?: boolean;
    target: string;
    type: "character" | "monster";
    vx: number;
    vy: number;
};
export declare type BankInfo = {
    [T in Exclude<BankPackType, "gold">]?: ItemInfo[];
} & {
    gold: number;
};
export declare type ChestInfo = PositionReal & {
    alpha: number;
    skin: "chest3" | string;
};
export declare type ItemInfo = {
    acc?: number;
    ach?: TitleName;
    b?: boolean;
    ex?: boolean;
    expires?: string;
    level?: number;
    name: ItemName;
    q?: number;
    p?: {
        chance: number;
        name: ItemName;
        level: number;
        scroll: ItemName;
        nums: number[];
    } | TitleName;
    ps?: TitleName[];
    rid?: string;
    stat_type?: StatType;
    v?: boolean;
};
export declare type SlotInfo = {
    [T in SlotType]: ItemInfo;
} & {
    [T in TradeSlotType]?: ItemInfo & {
        giveaway?: number;
        list: string[];
        price: number;
        rid: string;
    };
};
export declare type DoorInfo = [number, number, number, number, MapName, number?, number?, string?, string?];
export declare type StatusInfo = {
    [T in ConditionName | "typing"]?: {
        ms: number;
    };
} & {
    burned?: {
        intensity: number;
        f: string;
    };
    coop?: {
        id: string;
        p: number;
    };
    mluck?: {
        f: string;
        strong: boolean;
    };
    monsterhunt?: {
        sn: string;
        c: number;
        id: MonsterName;
    };
    citizen0aura?: {
        luck: number;
    };
    citizen4aura?: {
        gold: number;
    };
};
export declare type SInfo = {
    [T in MonsterName]?: IPosition & {
        map: string;
        live: boolean;
        hp: number;
        max_hp: number;
        target?: string;
    };
} & {
    valentines?: boolean;
};
export declare type PositionReal = IPosition & {
    map: MapName;
    real_x?: number;
    real_y?: number;
};
export declare type PositionMovable = PositionReal & {
    from_x?: number;
    from_y?: number;
    going_x: number;
    going_y: number;
};
export declare type PositionSmart = IPosition & {
    map: MapName;
    transport?: boolean;
    i?: number;
    s?: number;
};
export declare type IPosition = {
    map?: MapName;
    x: number;
    y: number;
};
export declare type CharacterType = "mage" | "merchant" | "paladin" | "priest" | "ranger" | "rogue" | "warrior";
export declare type DamageType = "magical" | "physical" | "pure";
export declare type ItemType = "box" | "cape" | "gem" | "material" | "misc" | "quest";
export declare type StatType = "armor" | "attack" | "blast" | "dex" | "dreturn" | "explosion" | "firesistance" | "for" | "frequency" | "fzresistance" | "gold" | "hp" | "int" | "lifesteal" | "luck" | "miss" | "mp_cost" | "mp_reduction" | "pnresistance" | "range" | "resistance" | "speed" | "str" | "stun" | "vit";
export declare type TitleName = "critmonger" | "festive" | "firehazard" | "glitched" | "gooped" | "legacy" | "lucky" | "shiny" | "sniper" | "stomped" | "superfast";
export declare type WeaponType = "axe" | "basher" | "bow" | "crossbow" | "dagger" | "dartgun" | "fist" | "mace" | "pmace" | "rapier" | "short_sword" | "spear" | "staff" | "stars" | "sword" | "wand" | "wblade";
export declare type MonsterName = "arcticbee" | "armadillo" | "bat" | "bbpompom" | "bee" | "bigbird" | "bluefairy" | "boar" | "booboo" | "bscorpion" | "cgoo" | "chestm" | "crab" | "crabx" | "croc" | "d_wiz" | "dknight2" | "dragold" | "eelemental" | "ent" | "felemental" | "fieldgen0" | "fireroamer" | "franky" | "frog" | "fvampire" | "gbluepro" | "ggreenpro" | "ghost" | "goblin" | "goldenbat" | "goo" | "gpurplepro" | "gredpro" | "greenfairy" | "greenjr" | "grinch" | "gscorpion" | "hen" | "iceroamer" | "jr" | "jrat" | "kitty1" | "kitty2" | "kitty3" | "kitty4" | "ligerx" | "mechagnome" | "minimush" | "mole" | "mrgreen" | "mrpumpkin" | "mummy" | "mvampire" | "nelemental" | "nerfedmummy" | "oneeye" | "osnake" | "phoenix" | "pinkgoblin" | "pinkgoo" | "plantoid" | "poisio" | "porcupine" | "pppompom" | "prat" | "puppy1" | "puppy2" | "puppy3" | "puppy4" | "rat" | "redfairy" | "rooster" | "rudolph" | "scorpion" | "skeletor" | "snake" | "snowman" | "spider" | "squig" | "squigtoad" | "stompy" | "stoneworm" | "target" | "target_a500" | "target_a750" | "target_ar500red" | "target_ar900" | "target_r500" | "target_r750" | "tinyp" | "tortoise" | "wabbit" | "welemental" | "wolf" | "wolfie" | "xscorpion";
export declare type BankPackType = "gold" | "items0" | "items1" | "items10" | "items11" | "items12" | "items13" | "items14" | "items15" | "items16" | "items17" | "items18" | "items19" | "items2" | "items20" | "items21" | "items22" | "items23" | "items24" | "items25" | "items26" | "items27" | "items28" | "items29" | "items3" | "items30" | "items31" | "items32" | "items33" | "items34" | "items35" | "items36" | "items37" | "items38" | "items39" | "items4" | "items40" | "items41" | "items42" | "items43" | "items44" | "items45" | "items46" | "items47" | "items5" | "items6" | "items7" | "items8" | "items9";
export declare type SlotType = "amulet" | "belt" | "cape" | "chest" | "earring1" | "earring2" | "elixir" | "gloves" | "helmet" | "mainhand" | "offhand" | "orb" | "pants" | "ring1" | "ring2" | "shoes";
export declare type TradeSlotType = "trade1" | "trade2" | "trade3" | "trade4" | "trade5" | "trade6" | "trade7" | "trade8" | "trade9" | "trade10" | "trade11" | "trade12" | "trade13" | "trade14" | "trade15" | "trade16";
export declare type ConditionName = "authfail" | "blink" | "burned" | "charging" | "charmed" | "dampened" | "darkblessing" | "easterluck" | "eburn" | "eheal" | "energized" | "fingered" | "frozen" | "fullguard" | "hardshell" | "holidayspirit" | "invincible" | "invis" | "licenced" | "marked" | "mcourage" | "mlifesteal" | "mluck" | "monsterhunt" | "mshield" | "notverified" | "phasedout" | "poisoned" | "poisonous" | "power" | "reflection" | "rspeed" | "sanguine" | "shocked" | "slowness" | "stack" | "stoned" | "stunned" | "sugarrush" | "tangled" | "warcry" | "weakness" | "withdrawal" | "xpower" | "xshotted";
export declare type ItemName = "5bucks" | "ale" | "amuletofm" | "angelwings" | "apiercingscroll" | "apologybox" | "armorbox" | "armorring" | "armorscroll" | "ascale" | "axe3" | "bandages" | "basher" | "basketofeggs" | "bataxe" | "bcandle" | "bcape" | "beewings" | "bfang" | "bfur" | "bkey" | "blade" | "blue" | "bottleofxp" | "bow" | "bow4" | "bowofthedead" | "bronzeingot" | "bronzenugget" | "brownegg" | "btusk" | "bugbountybox" | "bunnyears" | "bunnyelixir" | "bwing" | "cake" | "candy0" | "candy0v2" | "candy0v3" | "candy1" | "candy1v2" | "candy1v3" | "candycane" | "candycanesword" | "candypop" | "cape" | "carrot" | "carrotsword" | "cclaw" | "cdarktristone" | "cdragon" | "charmer" | "chrysalis0" | "claw" | "coal" | "coat" | "coat1" | "cocoon" | "computer" | "confetti" | "cosmo0" | "cosmo1" | "cosmo2" | "cosmo3" | "cosmo4" | "crabclaw" | "critscroll" | "crossbow" | "cscale" | "cscroll0" | "cscroll1" | "cscroll2" | "cscroll3" | "cshell" | "ctristone" | "cupid" | "cxjar" | "dagger" | "daggerofthedead" | "darktristone" | "dartgun" | "dexamulet" | "dexbelt" | "dexearring" | "dexearringx" | "dexring" | "dexscroll" | "dkey" | "dragondagger" | "drapes" | "dreturnscroll" | "dstones" | "ecape" | "ectoplasm" | "eears" | "egg0" | "egg1" | "egg2" | "egg3" | "egg4" | "egg5" | "egg6" | "egg7" | "egg8" | "eggnog" | "electronics" | "elixirdex0" | "elixirdex1" | "elixirdex2" | "elixirint0" | "elixirint1" | "elixirint2" | "elixirluck" | "elixirstr0" | "elixirstr1" | "elixirstr2" | "elixirvit0" | "elixirvit1" | "elixirvit2" | "emptyheart" | "emptyjar" | "epyjamas" | "eslippers" | "espresso" | "essenceofether" | "essenceoffire" | "essenceoffrost" | "essenceofgreed" | "essenceoflife" | "essenceofnature" | "evasionscroll" | "exoarm" | "fallen" | "fcape" | "fclaw" | "feather0" | "fieldgen0" | "fierygloves" | "figurine" | "fireblade" | "firebow" | "firecrackers" | "firestaff" | "firestars" | "flute" | "forscroll" | "frankypants" | "frequencyscroll" | "frogt" | "frostbow" | "froststaff" | "frozenstone" | "fsword" | "ftrinket" | "funtoken" | "fury" | "gbow" | "gcape" | "gem0" | "gem1" | "gem2" | "gem3" | "gemfragment" | "ghatb" | "ghatp" | "gift0" | "gift1" | "glitch" | "gloves" | "gloves1" | "goldbooster" | "goldenegg" | "goldenpowerglove" | "goldingot" | "goldnugget" | "goldring" | "goldscroll" | "gphelmet" | "greenbomb" | "gslime" | "gum" | "hammer" | "handofmidas" | "harbringer" | "harmor" | "hboots" | "hbow" | "hdagger" | "heartwood" | "helmet" | "helmet1" | "hgloves" | "hhelmet" | "hotchocolate" | "hpamulet" | "hpants" | "hpbelt" | "hpot0" | "hpot1" | "hpotx" | "iceskates" | "ijx" | "ink" | "intamulet" | "intbelt" | "intearring" | "intring" | "intscroll" | "jacko" | "jewellerybox" | "kitty1" | "lantern" | "lbelt" | "leather" | "ledger" | "licence" | "lifestealscroll" | "lostearring" | "lotusf" | "lspores" | "luckbooster" | "luckscroll" | "luckyt" | "mace" | "maceofthedead" | "mageshood" | "manastealscroll" | "mbones" | "mcape" | "mcarmor" | "mcboots" | "mcgloves" | "mchat" | "mcpants" | "mearring" | "merry" | "mistletoe" | "mittens" | "mmarmor" | "mmgloves" | "mmhat" | "mmpants" | "mmshoes" | "molesteeth" | "monsterbox" | "monstertoken" | "mparmor" | "mpcostscroll" | "mpgloves" | "mphat" | "mpot0" | "mpot1" | "mpotx" | "mppants" | "mpshoes" | "mpxamulet" | "mpxbelt" | "mpxgloves" | "mrarmor" | "mrboots" | "mrgloves" | "mrhood" | "mrnarmor" | "mrnboots" | "mrngloves" | "mrnhat" | "mrnpants" | "mrpants" | "mshield" | "mushroomstaff" | "mwarmor" | "mwboots" | "mwgloves" | "mwhelmet" | "mwpants" | "mysterybox" | "networkcard" | "nheart" | "northstar" | "offering" | "offeringp" | "offeringx" | "oozingterror" | "orbg" | "orbofdex" | "orbofint" | "orbofsc" | "orbofstr" | "orbofvit" | "ornament" | "ornamentstaff" | "outputscroll" | "pants" | "pants1" | "partyhat" | "phelmet" | "pico" | "pinkie" | "placeholder" | "placeholder_m" | "platinumingot" | "platinumnugget" | "pleather" | "pmace" | "poison" | "poker" | "powerglove" | "pstem" | "pumpkinspice" | "puppy1" | "puppyer" | "pvptoken" | "pyjamas" | "qubics" | "quiver" | "rabbitsfoot" | "rapier" | "rattail" | "redenvelope" | "redenvelopev2" | "redenvelopev3" | "rednose" | "reflectionscroll" | "resistancering" | "resistancescroll" | "rfangs" | "rfur" | "ringofluck" | "ringsj" | "rpiercingscroll" | "sanguine" | "santasbelt" | "scroll0" | "scroll1" | "scroll2" | "scroll3" | "scroll4" | "scythe" | "seashell" | "shadowstone" | "shield" | "shoes" | "shoes1" | "slimestaff" | "smoke" | "smush" | "snakefang" | "snakeoil" | "snowball" | "snowboots" | "snowflakes" | "snring" | "solitaire" | "spear" | "spearofthedead" | "speedscroll" | "spidersilk" | "spores" | "sshield" | "sstinger" | "staff" | "staff2" | "staff3" | "staff4" | "staffofthedead" | "stand0" | "stand1" | "starkillers" | "stealthcape" | "stick" | "stinger" | "stonekey" | "stoneofgold" | "stoneofluck" | "stoneofxp" | "storagebox" | "stramulet" | "strbelt" | "strearring" | "strring" | "strscroll" | "suckerpunch" | "supermittens" | "svenom" | "swifty" | "swirlipop" | "sword" | "swordofthedead" | "t2bow" | "t2dexamulet" | "t2intamulet" | "t2quiver" | "t2stramulet" | "t3bow" | "talkingskull" | "test" | "test2" | "test_orb" | "throwingstars" | "tracker" | "trinkets" | "tristone" | "troll" | "tshell" | "tshirt0" | "tshirt1" | "tshirt2" | "tshirt3" | "tshirt4" | "tshirt6" | "tshirt7" | "tshirt8" | "tshirt88" | "tshirt9" | "ukey" | "vblood" | "vitearring" | "vitring" | "vitscroll" | "wand" | "warmscarf" | "warpvest" | "watercore" | "wattire" | "wbasher" | "wblade" | "wbook0" | "wbook1" | "wbreeches" | "wcap" | "weaponbox" | "weaver" | "wgloves" | "whiskey" | "whiteegg" | "wine" | "wingedboots" | "woodensword" | "wshield" | "wshoes" | "x0" | "x1" | "x2" | "x3" | "x4" | "x5" | "x6" | "x7" | "x8" | "xarmor" | "xboots" | "xbox" | "xgloves" | "xhelmet" | "xmace" | "xmashat" | "xmaspants" | "xmasshoes" | "xmassweater" | "xpants" | "xpbooster" | "xpscroll" | "xptome" | "xshield" | "xshot";
export declare type MapName = "abtesting" | "arena" | "bank" | "bank_b" | "bank_u" | "batcave" | "cave" | "cgallery" | "cyberland" | "d2" | "d_a1" | "d_a2" | "d_b1" | "d_e" | "d_g" | "desertland" | "duelland" | "dungeon0" | "goobrawl" | "halloween" | "hut" | "jail" | "level1" | "level2" | "level2e" | "level2n" | "level2s" | "level2w" | "level3" | "level4" | "main" | "mansion" | "mansion_u" | "mtunnel" | "old_bank" | "old_main" | "original_main" | "resort" | "resort_e" | "shellsisland" | "ship0" | "spookytown" | "tavern" | "test" | "tunnel" | "winter_cave" | "winter_inn" | "winter_inn_rooms" | "winterland" | "woffice";
export declare type NPCRole = "announcer" | "blocker" | "bouncer" | "citizen" | "companion" | "compound" | "craftsman" | "cx" | "daily_events" | "exchange" | "funtokens" | "gold" | "guard" | "items" | "jailer" | "locksmith" | "lostandfound" | "lottery" | "mcollector" | "merchant" | "monstertokens" | "newupgrade" | "newyear_tree" | "petkeeper" | "premium" | "pvp_announcer" | "pvptokens" | "quest" | "repeater" | "resort" | "santa" | "secondhands" | "shells" | "ship" | "shrine" | "standmerchant" | "tavern" | "tease" | "thesearch" | "transport" | "witch";
export declare type NPCType = "antip2w" | "appearance" | "armors" | "basics" | "bean" | "beans" | "bouncer" | "citizen0" | "citizen1" | "citizen10" | "citizen11" | "citizen12" | "citizen13" | "citizen14" | "citizen15" | "citizen2" | "citizen3" | "citizen4" | "citizen5" | "citizen6" | "citizen7" | "citizen8" | "citizen9" | "compound" | "craftsman" | "exchange" | "fancypots" | "firstc" | "fisherman" | "funtokens" | "gemmerchant" | "goldnpc" | "guard" | "holo" | "holo0" | "holo1" | "holo2" | "holo3" | "holo4" | "holo5" | "items0" | "items1" | "items10" | "items11" | "items12" | "items13" | "items14" | "items15" | "items16" | "items17" | "items18" | "items19" | "items2" | "items20" | "items21" | "items22" | "items23" | "items24" | "items25" | "items26" | "items27" | "items28" | "items29" | "items3" | "items30" | "items31" | "items32" | "items33" | "items34" | "items35" | "items36" | "items37" | "items38" | "items39" | "items4" | "items40" | "items41" | "items42" | "items43" | "items44" | "items45" | "items46" | "items47" | "items5" | "items6" | "items7" | "items8" | "items9" | "jailer" | "leathermerchant" | "lichteaser" | "locksmith" | "lostandfound" | "lotterylady" | "mcollector" | "mistletoe" | "monsterhunter" | "newupgrade" | "newyear_tree" | "ornaments" | "pete" | "pots" | "premium" | "princess" | "pvp" | "pvpblocker" | "pvptokens" | "pwincess" | "rewards" | "santa" | "scrolls" | "secondhands" | "shellsguy" | "ship" | "shrine" | "standmerchant" | "tavern" | "tbartender" | "thief" | "transporter" | "wbartender" | "weapons" | "witch" | "wizardrepeater" | "wnpc";
export declare type ServerIdentifier = "HARDCORE" | "I" | "II" | "III" | "PVP";
export declare type ServerRegion = "ASIA" | "US" | "EU";
export declare type SkillName = "3shot" | "4fingers" | "5shot" | "absorb" | "agitate" | "alchemy" | "attack" | "blink" | "burst" | "cburst" | "charge" | "charm" | "cleave" | "curse" | "darkblessing" | "energize" | "entangle" | "esc" | "gm" | "hardshell" | "heal" | "huntersmark" | "interact" | "invis" | "light" | "magiport" | "mcourage" | "mentalburst" | "mluck" | "move_down" | "move_left" | "move_right" | "move_up" | "mshield" | "multi_burn" | "multi_freeze" | "open_snippet" | "partyheal" | "pcoat" | "phaseout" | "piercingshot" | "poisonarrow" | "portal" | "power" | "pure_eval" | "quickpunch" | "quickstab" | "reflection" | "regen_hp" | "regen_mp" | "revive" | "rspeed" | "scare" | "self_healing" | "selfheal" | "shadowstrike" | "snippet" | "snowball" | "stack" | "stomp" | "stone" | "stop" | "supershot" | "tangle" | "taunt" | "throw" | "toggle_character" | "toggle_code" | "toggle_inventory" | "toggle_run_code" | "toggle_stats" | "track" | "travel" | "use_hp" | "use_mp" | "use_town" | "warcry" | "warp" | "xpower";
